#!/usr/bin/env python

#    This file is part of VASM.
#
#    VASM is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License v3 as published by
#    the Free Software Foundation.
#
#    VASM is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VASM.  If not, see <http://www.gnu.org/licenses/>.

__author__ = "Moises Henriquez"
__author_email__ = "moc.liamg@xnl.E0M"[::-1]

"""FSTAB.py
Provides a data model for working with /etc/fstab."""

import parted
import os
import utils

def list_all_system_partitions():
    """ return a list of all partition paths"""
    partitions = []
    for dev in parted.getAllDevices():
        if not dev.readOnly and dev.getSize() > 0.0:
           dsk = parted.Disk(dev)
           for part in dsk.partitions:
               # extended partitions are type 2 and have no fs
               if part.type != 2:
                   partitions.append(part.path)
    return partitions

def list_all_system_drives():
    """ Return a list of all device drive paths """
    drives = [dev.path for dev in parted.getAllDevices() if not dev.readOnly and dev.getSize() > 0.0]
    return drives

class FstabEntry(object):
    """ Object representing an entry in /etc/fstab """
    def __init__(self, device = None):
        self.device = device
        self.uuid = None
        self.mountpoint = None
        self.filesystem = None
        self.options = None
        self._mount_now = True
    
    def find_default_mountpoint(self):
        assert self.device is not None, "device path must be set first."
        assert self.filesystem is not None, "Unknown filesystem.  Set filesystem property first."
        ret = None
        if 'swap' in self.filesystem:
            ret = 'swap'
        else:
            ret = self.device.replace('/dev/','/mnt/')
        return ret

    def find_default_options(self):
        """ Returns the default fstab options for the partitions filesystem """
        assert self.filesystem is not None, "Unknown filesystem."
        opts = {
            "ext2":"defaults 0 0",
            "ext3":"defaults 0 0",
            "ext4":"defaults 0 0",
            "iso9660":"noauto,ro,user 0 0",
            "msdos":"defaults 0 0",
            "swap":"sw 0 0",
            "linux-swap":"sw 0 0",
            "linux-swap(v1)":"sw 0 0"
        }
        if opts.has_key(self.filesystem):
            return opts[self.filesystem]
        elif 'swap' in self.filesystem:
            return opts['swap']
        else:
            return 'defaults 0 0'

    def find_uuid(self):
        """ Find the uuid for the specified device """
        assert self.device is not None, "You must specify a device node first."
        #assert "/" not in self.device, "Invalid device node."
        uuiddir = "/dev/disk/by-uuid"
        partition = os.path.split(self.device)[-1]
        for entry in os.listdir(uuiddir):
            lname = os.readlink(os.path.join(uuiddir, entry))
            if lname.endswith(partition):
                return entry
        return None

    def find_path(self):
        """ Find the partition path.  This is useful if fstab provides UUID but not path """
        assert self.uuid is not None, "Cannot find path if uuid is not known"
        uuiddir = "/dev/disk/by-uuid"
        for uid in os.listdir(uuiddir):
            if uid == self.uuid:
                lname = os.readlink(os.path.join(uuiddir, uid))
                return os.path.join("/dev/", lname.split("/")[-1])
        return None

    def _activate_swap(self):
        """ Activate a swap partition after it has been added """
        return utils.activate_swap(self.device)

    def mount(self):
        """ Mount this entry to its mountpoint """
        assert self.device not in ("",None), "No partition has been specified.  Cannot mount"
        assert self.mountpoint not in ("",None), "No mountpoint has been specified for this partition."
        assert self.options not in ("",None), "No mounting options specified"
        assert self.filesystem not in ("",None), "No filesystem specified"
        # We want to catch this exception, because the partition may already 
        # be mounted.
        if "swap" in self.filesystem:
            try:
                self._activate_swap()
                return
            except:
                pass
        try:
            utils.mount(self.device, self.mountpoint)
        except:
            pass
        return
        
    def find_filesystem(self):
        """ Find the partition's filesystem as reported by parted """
        assert self.device is not None, "You must specify a device node first."
        devicename = self.device[:len("/dev/sda")]
        pdev = parted.Device(devicename)
        pdisk = parted.Disk(pdev)
        ppart = [p for p in pdisk.partitions if p.path == self.device ][0]
        if "swap" in ppart._fileSystem.type:
            ret = "swap"
        else:
            ret = ppart._fileSystem.type

        return ret

class FstabModel(object):
    """ Data model for working with /etc/fstab """
    def __init__(self, dataobject = '/etc/fstab'):
        self.observers = []
        self.dataobject = dataobject
    
    def notify(self):
        """ Notify all observers that there have been changes """
        for f in self.observers:
            f()
    
    def add_observer(self, method):
        """ Add an observer to the model"""
        self.observers.append(method)
        return

    def removeEntry(self, device_path=""):
        """ Remove the fstab entry with the device_path. """
        assert device_path not in ("", None), "Invalid device path"
        assert self.hasEntry(device_path), "%s is not listed in %s"% (device_path, self.dataobject)
        ndata = []
        with open(self.dataobject) as data:
            for line in data:
                if len(line.strip()) == 0:
                    continue
                if not line.startswith("#"):
                    entry = self.get_entry_from_line(line)
                    if entry.device == device_path:
                        continue
                ndata.append(line)
        f = open(self.dataobject, 'w')
        f.writelines(''.join(ndata))
        f.close()
        return self.notify()

    def addEntry(self, entry):
        """ Add a new entry to fstab.
        Arguments:
            entry - An FstabEntry() object """
        assert isinstance(entry, FstabEntry), "entry argument is of invalid type"
        assert hasattr(entry, 'mountpoint'), "Missing mountpoint attribute."
        sep = " "*4
        if entry.uuid is None:
            entry.uuid = entry.find_uuid()
        if entry.filesystem is None:
            entry.filesyatem = entry.find_filesystem()
        # Use UUID when possible
        if entry.uuid is None:
            part = entry.device
        else:
            part = "UUID=%s"% entry.uuid

        if entry.options is None:
            options = entry.find_default_options()
        else:
            options = entry.options
        data = []
        with open(self.dataobject) as f:
            for line in f:
                data.append(line)
        line = sep.join((
            part,
            entry.mountpoint,
            entry.filesystem,
            options
            ))
        data.append(line + "\n")
        f = open(self.dataobject, 'w')
        f.writelines(''.join(data))
        f.close()
        return self.notify()

    def listEntries(self):
        """ Return a list of FstabEntry objects representing each entry in fstab """
        ret = []
        sep = " "*4
        with open(self.dataobject) as f:
            for line in f:
                if not line.strip():
                    continue
                if line.startswith("#"):
                    continue
                entry = self.get_entry_from_line(line)
                ret.append(entry)
        return ret

    def getEntry(self, device_path=""):
        """ Get an entry object for an existing fstab entry."""
        assert device_path not in ('', None), \
            "You must specify a device path to get the existing entry object."
        for item in self.listEntries():
            if item.device == device_path:
                return item
        return None

    def get_entry_from_line(self, line):
        """ Create a valid entry from the provide line """
        sp = line.split()
        sep = " "*4
        entry = FstabEntry()
        dev = sp[0].strip()
        if dev.startswith("UUID="):
            entry.uuid = dev.split("=")[-1].strip()
            entry.device = entry.find_path()
        else:
            entry.device = dev.strip()
            entry.uuid = entry.find_uuid()
        entry.mountpoint = sp[1].strip()
        entry.filesystem = sp[2].strip()
        entry.options = sep.join(sp[3::]).strip()

        return entry

    def hasEntry(self, device_path=""):
        """ Check if fstab has an entry for device_path """
        assert device_path not in ("",None), "Invalid device path"
        entries = self.listEntries()
        for item in entries:
            if item.device == device_path:
                return True

        return False
