#!/usr/bin/env python

#    This file is part of VASM.
#
#    VASM is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License v3 as published by
#    the Free Software Foundation.
#
#    VASM is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VASM.  If not, see <http://www.gnu.org/licenses/>.

__author__ = "Moises Henriquez"
__author_email__ = "moc.liamg@xnl.E0M"[::-1]

import gtk
import sys
import os
from vasm.ui.gtk2.support import dialogs, widgets
from vasm.backend.utils import _, datapath
from vasm.backend import BOOTMODE
import logging

logger = logging.getLogger('vasm')

# vasm flags
VASM_CATEGORY = "%s/%s"% (_("System"), _("Startup Options"))
VASM_ICON = os.path.join(datapath, 'bootmode.png')
VASM_LABEL=_("Startup Mode")

def __vasm_test__():
    """ Tests to decide who sees this module in vasm... Must return True | False"""
    return os.geteuid() == 0

class VectorlinuxBootMode(widgets.VasmModule):
    def __init__(self, parent):
        self._parent = parent
        title = VASM_LABEL
        self.datamodel = BOOTMODE.InitTab()
        desc = _("These settings define how your system starts.") + "  " +\
        _("You may choose to start your system in text mode rather than a graphical environment.") + \
        "  " + _("Select how the system should start from the choices below.")
        widgets.VasmModule.__init__(self, parent=parent,
                                    module_title=title, module_desc = desc)
        
        # add the options
        tuibox = gtk.HBox()
        tuiimg = gtk.Image()
        self.tuirb = gtk.RadioButton(
            label = _("Text User Interface (command prompt)")
        )
        tuiimg.set_from_pixbuf(
            gtk.gdk.pixbuf_new_from_file_at_size(
                os.path.join(datapath, 'terminal.png'), 48, 48
            )
        )
        tuibox.pack_start(tuiimg, False, False, 24)
        tuibox.pack_start(self.tuirb, False, True, 4)
        
        guibox = gtk.HBox()
        guiimg = gtk.Image()
        self.guirb = gtk.RadioButton(group=self.tuirb,
            label = _("Graphical Desktop Interface"))
        guiimg.set_from_pixbuf(
            gtk.gdk.pixbuf_new_from_file_at_size(
                os.path.join(datapath, 'gui_desktop.png'), 48, 48
            )
        )
        guibox.pack_start(guiimg, False, False, 24)
        guibox.pack_start(self.guirb, False, True, 4)
        # multi user enable/disable
        self.multiusercb = gtk.CheckButton(
            label=_("Allow multiple users to log in simultaneusly"))
        multiuserbox = gtk.HBox()
        multiuserbox.pack_start(self.multiusercb, False, True, 2)

        # add options to the module
        self.pack_start(tuibox, False, False, 4)
        self.pack_start(guibox, False, False, 4)
        # add the checkbox for multi user login
        self.pack_start(multiuserbox, False, False, 4)

        # display the current settings on the screen
        self.show_current_settings()
        # add a label saying this needs a reboot
        lbox = gtk.HBox()
        notice = _(
            "Note: Changes will be reflected after the system restarts.")
        lb = widgets.DLabel(notice)
        lbox.pack_start(lb, True, True, 4)
        self.pack_start(lbox, False, True, 18)

        # the action area
        btclose = gtk.Button(stock=gtk.STOCK_CANCEL)
        btclose.connect('clicked', self.close_module)
        self.add_button(btclose)
        
        btapply = gtk.Button(stock=gtk.STOCK_APPLY)
        btapply.connect('clicked', self.do_apply_bootmode)
        self.add_button(btapply)
    
    def do_apply_bootmode(self, widget=None):
        """ save changes to the system """
        value = self._get_selected_initvalue()
        try:
            self.datamodel.setMode(value)
            logger.info("Bootmode set to %s"% value)
        except IOError, (errorno, errormsg):
            if errorno == 13:
                logger.error("Error while setting bootmode. %s"%errormsg)
                return dialogs.error(
                    _("Unable to gain write permissions to /etc/inittab."),
                    parent=self._parent)
        return dialogs.info(_(
            "The default boot method has been changed to init level") + " " + value +\
           ".  " + _( "The changes will be reflected after the computer restarts."),
            parent=self._parent)

    def show_current_settings(self):
        """ Adjust the graphical widgets to display the current mode reported
        by the backend """
        current = self.datamodel.getMode()
        if current == 2:
            self.tuirb.set_active(True)
            self.multiusercb.set_active(False)
        elif current == 3:
            self.tuirb.set_active(True)
            self.multiusercb.set_active(True)
        elif current == 4:
            self.guirb.set_active(True)
            self.multiusercb.set_active(False)
        elif current == 5:
            self.guirb.set_active(True)
            self.multiusercb.set_active(True)

    def _get_selected_initvalue(self):
        """ Returns the selected init value as a string that needs to
        be parsed to BOOTMODE.set_boot_mode
        """
        val = 0
        if self.tuirb.get_active():
            val = 2
        elif self.guirb.get_active():
            val = 4
        
        if self.multiusercb.get_active():
            val += 1
        return str(val)


def __run__(parent):
    ui = VectorlinuxBootMode(parent)
    return ui.launch()

