#!/usr/bin/env python

#    This file is part of VASM.
#
#    VASM is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License v3 as published by
#    the Free Software Foundation.
#
#    VASM is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VASM.  If not, see <http://www.gnu.org/licenses/>.

__author__ = "Moises Henriquez"
__author_email__ = "moc.liamg@xnl.E0M"[::-1]

import sys
import gtk
import os
from vasm.ui.gtk2.support import dialogs, widgets
from vasm.backend import DATESET
from vasm.backend.utils import _, datapath
import logging

logger = logging.getLogger('vasm')

class DateTimeDialog(widgets.VasmModule):
    def __init__(self, parent):
        title = _("System date and time")
        description = _("Adjust the system date and time setting using the controls below.")
        widgets.VasmModule.__init__(self, parent=parent,
                                    module_title = title,
                                    module_desc = description)
        # The models
        self.TZmodel = DATESET.TimeZone()
        self.HWClockModel = DATESET.HWClock()
        self.DateModel = DATESET.DateTime()
        
        # add the calendar
        self.calendar = gtk.Calendar()
        calbox = gtk.HBox()
        calbox.pack_start(self.calendar, False, True, 2)
        self.pack_start(calbox, False, False, 4)
        

        # Add a clock below the calendar.
        clockframe = gtk.Frame(_("System time"))
        clockbox = gtk.VBox()
        self.clock = widgets.WideTimePicker()
        clockbox.pack_start(self.clock, False, True, 8)
        clockframe.add(clockbox)
        self.pack_start(clockframe, False, True, 4)

        # hardware clock setting.
        hwclockframe = gtk.Frame(_("Hardware Clock"))
        hwbox = gtk.VBox()
        hwclockframe.add(hwbox)
        msg = (_("Linux uses the hardware clock setting and timezone to properly set the system time."),
               " ",
               _("To get the propper results, select the value that matches your hardware (BIOS) clock setting."),
            _("If you are unsure about this, 'Localtime' is the common value for most PC's."))
        lb = widgets.DLabel(' '.join(msg))
        self.hwclockopts = widgets.HOptionsBox(
            label = "<b>%s</b>"% _("Hardware Clock Setting:") + " ",
            options = ['UTC', 'Localtime']
            )
        # Set the default value for this choicebox
        self.hwclockopts.set_value(
            self.HWClockModel.get_value()
            )
        hwbox.pack_start(lb, False, True, 4)
        hwbox.pack_start(self.hwclockopts, False, True, 8)
        # The timezone picker
        zoneoptions = self.TZmodel.list_zones()
        self.tzselector = widgets.DropdownBox(
            label="<b>%s </b>"% _("Select Timezone:"),
            choices = zoneoptions
            )
        # set the current value on the dropdown list
        self.tzselector.set_value(
            self.TZmodel.get_zone()
            )
        hwbox.pack_start(self.tzselector, False, True, 8)
        # Add the content to the module body
        self.pack_start(hwclockframe, False, True, 4)
        
        # module action buttons
        btclose = gtk.Button(stock=gtk.STOCK_CANCEL)
        btclose.connect('clicked', self.close_module)
        self.add_button(btclose)
        bt_apply = gtk.Button(stock=gtk.STOCK_APPLY)
        bt_apply.connect('clicked', self.apply_event)
        self.add_button(bt_apply)

    def _set_date(self, widget=None):
        """ Save the date settings to the system."""
        _year, _month, _day = self.calendar.get_date()
        # gtk.Calendar's moth array starts with 0, so we need to add 1 to the returned value
        self.DateModel.year = _year
        self.DateModel.month = _month + 1
        self.DateModel.day = _day
        return self.DateModel.set_date()

    def _set_time(self, widget=None):
        """ Save the time settings to the system. """
        _hr, _min, _sec = self.clock.get_value()
        self.DateModel.hour = _hr
        self.DateModel.minute = _min
        self.DateModel.second = _sec
        return self.DateModel.set_time()

    def _set_hwclock_setting(self, widget=None):
        """ Save the hardware clock setting to the system. """
        hwc = self.hwclockopts.get_value()
        return self.HWClockModel.set_value(hwc)

    def _set_timezone(self, widget=None):
        """ Save the timezone setting to the system."""
        zone = self.tzselector.get_value()
        return self.TZmodel.set_zone(zone)

    def apply_event(self, widget=None):
        """ Save all settings. """
        try:
            self._set_date()
            logger.debug("Saved date settings.")
        except DATESET.DateTimeError as e:
            logger.error("Error saving date settings. %s"% e)
            return dialogs.error(
                "%s \n\n%s"% (
                    _("Failed to set the sytem date."),
                    e
                    )
                )
        try:
            self._set_time()
            logger.debug("Saved time settings.")
        except DATESET.DateTimeError as e:
            logger.error("Error while saving system time settings. %s"% e)
            return dialods.error(
                "%s \n\n%s"% (
                    _("Failed to set the system time."), e
                    )
                )

        try:
            self._set_hwclock_setting()
            logger.debug("Saved system hardware clock settings.")
        except IOError as e:
            # unable to lock the file
            logger.error("Unable to save hardware clock settings. %s"% e)
            return dialogs.error(
                "%s \n\n%s"%
                (_("Unable to save hardware clock setting."), e)
                )
        except AssertionError as e:
            logger.error("Unable to save hardware clock settings. %s"% e)
            return dialogs.error(
                _("Unable to save the hardware clock setting") + \
                "\n\n" % e
                )
        try:
            self._set_timezone()
            logger.debug("Saved timezone settings")
        except IOError as e:
            logger.error("Unable to save timezone setting. %s"% e)
            return dialogs.error(
                "%s \n\n %s"% (
                    _("Unable to save timezone setting."), e
                    )
                )

        dialogs.info(
            _("Settings successfully saved.")
            )
        return self.close_module()

def __vasm_test__():
    """ Make this available to root only """
    return os.geteuid() == 0

def __run__(parent=None):
    setter = DateTimeDialog(parent=parent)
    return setter.launch()


VASM_CATEGORY = _("System")
VASM_ICON = os.path.join(datapath, 'datetime.png')
VASM_LABEL=_("Date / Time Settings")

