#!/usr/bin/env python

#    This file is part of VASM.
#
#    VASM is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License v3 as published by
#    the Free Software Foundation.
#
#    VASM is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VASM.  If not, see <http://www.gnu.org/licenses/>.

import gtk
import os
from vasm.ui.gtk2.support import widgets, dialogs
from vasm.backend.utils import _, datapath
from vasm.backend import LOGINMANAGERS
import logging

logger = logging.getLogger('vasm')

__author__ = "Moises Henriquez"
__author_email__ = "moc.liamg@xnl.E0M"[::-1]

#vasm flags
VASM_CATEGORY="%s/%s"% (_("System"), _("Graphical Environment"))
VASM_LABEL=_("Login Manager Settings")
VASM_ICON = os.path.join(datapath, 'gksu.png')


def __vasm_test__():
    """ run any tests required for this module to be shown in the mian vasm UI
    
    Must return True or False"""
    return not os.geteuid() > 0

class GtkLoginManager(widgets.VasmModule):
    def __init__(self, parent):
        exp = (
            _("The login manager allows you to log into your system and select"),
            _("your desktop session.")," ",
            _("Select your preferred login manager from the options below."), " ",
            _("Click 'Apply' when done.")
        )
        
        widgets.VasmModule.__init__(self, parent, VASM_LABEL, ' '.join(exp))
        
        # box in the middle
        midbox = gtk.HBox()
        # Treeview
        model = gtk.ListStore(str)
        # Get the list of login managers and show them on the treeview
        for loginmanager in LOGINMANAGERS.LoginManagers().list_existing():
            model.append([loginmanager])

        listbox = gtk.TreeView(model=model)
        listbox.connect('cursor-changed', self.update_selection)
        col = gtk.TreeViewColumn()
        col.set_property('min-width', 120)
        listbox.set_headers_visible(False)
        crend = gtk.CellRendererText()
        col.pack_start(crend)
        col.add_attribute(crend, 'text', 0)
        listbox.append_column(col)
        midbox.pack_start(listbox, False, True, 4)

        # preview box
        self.preview = gtk.Image()
        midbox.pack_start(self.preview, True, True, 12)
        
        # add the content to the module
        self.pack_start(midbox, True, True, 4)
        
        # Display a preview for the current login manager.
        current = LOGINMANAGERS.LoginManagers().get_current().lower()   
        self._show_preview(current)
        
        # buttons for the action area
        self.add_button((gtk.Button(stock=gtk.STOCK_CANCEL), self.close_module))
        self.add_button((gtk.Button(stock=gtk.STOCK_APPLY), self.apply_changes))

    def _show_preview(self, manager):
        """ Display the preview for the current login manager """
        fname = "login_%s.png"%manager
        fpath = os.path.join(datapath, fname)
        if os.path.exists(fpath):
            self.preview.set_from_pixbuf(
                gtk.gdk.pixbuf_new_from_file_at_size(
                    fpath, 256, 256
                )
            )
        else:
            self.preview.set_from_pixbuf(
                gtk.gdk.pixbuf_new_from_file_at_size(
                    os.path.join(filepath, "login_unknown.png"), 256, 256
                )
            )

    def apply_changes(self, widget=None):
        """ Save changes to the system """
        manager = self.selected_loginmanager.lower()
        # The backend will respond with True if it goes trhough, False otherwise.
        try:
            LOGINMANAGERS.LoginManagers().set_manager(manager)
            logger.debug("System login manager set to %s"% manager)
            msg = (
                _("The system login manager has been set to"), " %s "% manager,
                ".  ", 
                _("You will see the changes applied when your system restarts."))
            return dialogs.info(''.join(msg))
        except AssertionError, e:
            logger.error("Error trying to save login manger %s"% e)
            msg = (
            _("Unable to save login manager settings."), e)
            return dialogs.error("".join(msg))
        except IOError, (errorno, errormsg):
            logger.error("Error trying to save login manager settings %s"% errormsg)
            msg = (_("Unable to save login manager settings."), errormsg)
            return dialogs.error("".join(msg))

    def update_selection(self, treeview):
        """ Update the selected login manager """
        col = treeview.get_cursor()[0][0]
        selection = treeview.get_model()[col][0].lower()
        self.selected_loginmanager = selection
        fname = "login_%s.png"% selection
        filepath = os.path.join(datapath, fname)
        if os.path.exists(filepath):
            self.preview.set_from_pixbuf(
                gtk.gdk.pixbuf_new_from_file_at_size(
                    filepath, 256, 256
                    )
                )
        else:
            self.preview.set_from_pixbuf(
                gtk.gdk.pixbuf_new_from_file_at_size(
                    os.path.join(datapath,"login_unknown.png"), 256, 256
                    )
                )
        

def __run__(parent):
    gui = GtkLoginManager(parent)
    return gui.launch()

