#!/usr/bin/env python

#    This file is part of VASM.
#
#    VASM is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License v3 as published by
#    the Free Software Foundation.
#
#    VASM is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VASM.  If not, see <http://www.gnu.org/licenses/>.

import gtk
import sys
import os
from vasm.ui.gtk2.support import widgets, dialogs
from vasm.backend.utils import _
from vasm.backend import PASSWORDS
import logging

logger = logging.getLogger('vasm')

__author__ = "Moises Henriquez"
__author_email__ = "moc.liamg@xnl.E0M"[::-1]


# VASM module constants
VASM_CATEGORY = "Personal"
VASM_LABEL="Change My Password"
VASM_ICON=gtk.STOCK_DIALOG_AUTHENTICATION

def __vasm_test__():
    """ Run any tests that must be met before this module can be made available
    in the main UI"""
    return True

class GtkUI(widgets.VasmModule):
    def __init__(self, parent, for_user):
        desc = ""
        self.for_user = for_user
        widgets.VasmModule.__init__(self, parent=parent,
                                    module_title = VASM_LABEL,
                                    module_desc = desc)
        
        # password fields
        self.oldpassbox = widgets.TextEntry("Enter old password", True, 18)
        self.newpass1 = widgets.TextEntry("Enter new password", True, 18)
        self.newpass2 = widgets.TextEntry("Verify new password", True, 18)
        
        for box in (self.oldpassbox, self.newpass1, self.newpass2):
            self.pack_start(box, False, False, 2)
        
        # connect the signal to detect password changes
        self.newpass1.entry.connect('changed', self.test_entry)
        self.newpass2.entry.connect('changed', self.test_entry)
        
        # the action buttons
        self.okbutton = gtk.Button(stock=gtk.STOCK_APPLY)
        self.okbutton.connect('clicked', self.do_apply_changes)
        btexit = gtk.Button(stock=gtk.STOCK_CANCEL)
        btexit.connect('clicked', self.close_module)
        self.add_button(btexit)
        self.add_button(self.okbutton)
        
        
        # set the module text according to who is using it
        if for_user == "own":
            if os.geteuid() > 0:
                # user can only change his or her password.
                name = os.getenv("USERNAME")
                self._successmsg = _("Your password has been successfully changed.")
                title = "<b>%s</b>"% _("Change my password.")
                tx = "%s"% (
                    _("Choose a password that is at least 6 characters long and consists of lower and upper case letters and numbers")
                )
                self.oldpassbox.entry.grab_focus()
            else:
                # root is changing the root password
                for_user = "root"
                self._successmsg = _("The root password on this system has been changed.")
                title = "<b>%s</b>"% _("Change administrator password")
                tx = "%s\n"% (
                    _("This is the linux root password which grants access to ALL parts of the system.") + \
                    "  " + \
                    _("Choose a password that is at least 6 characters long and consists of") + \
                    " " + _("lower and upper case letters and numbers.")
                )
                self.oldpassbox.entry.set_property('sensitive', False)
                self.newpass1.entry.grab_focus()
            
        else:
            self._successmsg = _("Successfully changed the user's password")
            title = "<b>%s</b>"% _("change password for") + " " + for_user
            tx = "%s %s. %s"% (_("You are changing the password for"), for_user,
            _("Enter new passwords for this user below"))
            self.oldpassbox.entry.set_property('sensitive', False)
            self.newpass1.entry.grab_focus()
        
        self.set_title(title)
        self.set_description(tx)
        self.okbutton.set_property('sensitive', False)

    def test_entry(self, textbox=None):
        if len(self.newpass1.get_text()) >= 5:
            if self.newpass1.get_text() == self.newpass2.get_text():
                self.okbutton.set_sensitive(True)
                return
        self.okbutton.set_sensitive(False)
    
    def clear_form(self):
        for box in self.oldpassbox.entry, self.newpass1.entry, self.newpass2.entry:
            box.set_text("")
        self.oldpassbox.entry.grab_focus()
    
    def do_apply_changes(self, widget=None):
        oldpass = self.oldpassbox.get_text()
        newpass = self.newpass1.get_text()
        
        try:
            PASSWORDS.change_password(self.for_user, oldpass, newpass)
            logger.debug("Changed password for %s"% self.for_user)
        except PASSWORDS.BadPasswordException:
            self.clear_form()
            msg = (
                "<b>%s</b>\n\n"% _("Incorrect Password"),
                _("The current password you entered is incorrect."), " ",
                _("Your password remains unchanged.")
            )
            logger.error("Unable to password for %s.  The current password is wrong."% self.for_user)
            return dialogs.error(' '.join(msg), title=_("Bad password"),
                                 parent = self._parent)
        except PASSWORDS.BadNewPasswordException:
            self.clear_form()
            msg = (
                "<b>%s</b>\n\n"% _("Bad Password."),
                _("The new password you provided does not meet the minimum requirements."),
                " ",
                _("Passwords should be at least 6 characters long and contain"),
                _("a mixture of upper and lower case letters and numbers.")
            )
            logger.error("Unable to change password for %s.  Bad new password."% self.for_user)
            return dialogs.error(' '.join(msg),
                                 parent = self._parent, title=_("Bad Password"))
        else:
            dialogs.info(self._successmsg, title=_("Password changed."),
                                parent = self._parent)
            return self.close_module()

    

def __run__(parent=None, user="own"):
    ui = GtkUI(parent, for_user=user)
    ui.launch()
    if os.geteuid() > 0:
        ui.oldpassbox.entry.grab_focus()
    else:
        ui.newpass1.entry.grab_focus()