#!/usr/bin/env python

#    This file is part of VASM.
#
#    VASM is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License v3 as published by
#    the Free Software Foundation.
#
#    VASM is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VASM.  If not, see <http://www.gnu.org/licenses/>.

__author__ = "Moises Henriquez"
__author_email__ = "moc.liamg@xnl.E0M"[::-1]

"""pyfstab.py
Provides an graphical way to interact with /etc/fstab"""

import os
import sys
import gtk
from vasm.ui.gtk2.support import dialogs, widgets
from vasm.backend import FSTAB
from vasm.backend.utils import _, datapath
import logging

logger = logging.getLogger('vasm')

# vasm flags

VASM_CATEGORY = "%s/%s"% (_("System"), _("Startup Options"))
VASM_LABEL=_("Mount Points")
VASM_ICON = os.path.join(datapath, 'harddisk.png')


def __vasm_test__():
    """ Set of tests to decide when to display this module
    """
    return os.geteuid() == 0

class FstabManager(widgets.VasmModule):
    def __init__(self, parent):
        self._parent = parent
        tx = _("Setup the partitions and devices that should be automatically mounted when the system starts up.")
        widgets.VasmModule.__init__(self, parent=self._parent,
                                    module_title=VASM_LABEL, module_desc=tx)
        
        self.datamodel = FSTAB.FstabModel()
        self.datamodel.add_observer(self.refresh_fstab_list)
        #make the treeview
        store = gtk.ListStore(str,str, str, str, str)
        self.fstree = gtk.TreeView(model=store)
        rendPart = gtk.CellRendererText()
        rendFsType = gtk.CellRendererText()
        rendMntPoint = gtk.CellRendererText()
        rendOptions = gtk.CellRendererText()
        rendUuid = gtk.CellRendererText()
        
        colUuid = gtk.TreeViewColumn("UUID", rendUuid, text=4)
        colPart = gtk.TreeViewColumn(_("Partition"), rendPart, text=0)
        colPart.set_min_width(100)
        colFsType = gtk.TreeViewColumn(_("Format"), rendFsType, text=1)
        colFsType.set_min_width(75)
        colMntPoint = gtk.TreeViewColumn(_("Mount Point"), rendMntPoint, text=2)
        colMntPoint.set_min_width(100)
        colOptions = gtk.TreeViewColumn(_("Mount Options"), rendOptions, text=3)
        
        # add columns to the tree
        for col in (colPart, colFsType, colMntPoint, colOptions, colUuid):
            col.set_resizable(True)
            self.fstree.append_column(col)
        
        # put the tree in a scrolled window
        sw = gtk.ScrolledWindow()
        sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        treebox = gtk.HBox()
        treebox.pack_start(self.fstree, True, True, 4)
        sw.add_with_viewport(treebox)
        
        # add the scroll to the box
        self.pack_start(sw, True, True, 4)
        
        #sw.add(self.fstree)
        
        # Show the current setup on the grid
        self.refresh_fstab_list()
    
        # the toolbar for adding and removing mountpoints.
        buttonbar = gtk.HBox()
        btAdd = gtk.Button(stock=gtk.STOCK_ADD)
        btAdd.connect("clicked", self.add_event)
        btRem = gtk.Button(stock=gtk.STOCK_DELETE)
        btRem.connect("clicked", self.delete_event)
        #btEdit = gtk.Button(stock=gtk.STOCK_EDIT)
        for bt in (btAdd, btRem):
            buttonbar.pack_start(bt, False, False, 2)
        self.pack_start(buttonbar, False, True, 4)
        #self.body.pack_start(buttonbar, False, True, 4)
        
        
        # the action area
        btclose = gtk.Button(stock=gtk.STOCK_CANCEL)
        btclose.connect('clicked', self.close_module)
        self.add_button(btclose)

    def delete_event(self, button=None):
        """ Remove the selected entry from fstab"""
        try:
            row = self.fstree.get_cursor()[0][0]
        except:
            dialogs.error(_("Nothing is selected.  Select what to delete"))
            return
        model = self.fstree.get_model()
        selected = model[row][0]
        ret = dialogs.question(
            _("Are you sure you want to remove this entry?"), parent=self._parent)
        if ret == gtk.RESPONSE_YES:
            try:
                self.datamodel.removeEntry(selected)
                logger.debug("Deleted %s from /etc/fstab"%selected)
            except AssertionError as e:
                logger.error("Unable to remove %s from /etc/fstab. %s"% (selected, e))
                return dialogs.error(
                    _("Unable to delete this entry") + "  \n\n" + e)
        # Simply return if the user backs out
        return
    
    def add_event(self, widget=None):
        w = FstabEntryDialog(parent=self._parent, fstabmodel=self.datamodel)
        w.set_title(_("Add new mounpoint to system startup"))
        res = w.run()
        if res == gtk.RESPONSE_OK:
            if not w.target:
                if w.targetbox.get_text() == "":
                    errtxt = _(("You did not specify a mount point for your entry.  "
                                "Incomplete entries cannot be processed"))
                    dialogs.error(txt=errtxt, title="Incomplete Entry", parent=self._parent)
                    #dialogs.error("%s")% errtxt
                    w.destroy()
                    return
            ret = w.do_add_entry()
            w.destroy()
            return ret
        w.destroy()

    def refresh_fstab_list(self):
        store = self.fstree.get_model()
        store.clear()
        for entry in self.datamodel.listEntries():
            uid = entry.uuid or "N/A"
            store.append([
                entry.device, entry.filesystem, entry.mountpoint, entry.options, uid
                ])
        

def __run__(parent):
    gui = FstabManager(parent)
    return gui.launch()


class FstabEntryDialog(gtk.Dialog):
    """ Dialog to collect information for a new entry to add to /etc/fstab """
    def __init__(self, parent=None, fstabmodel=None):
        gtk.Dialog.__init__(self, parent=parent)
        self.set_has_separator(True)
        self.add_button(gtk.STOCK_SAVE, gtk.RESPONSE_OK)
        self.add_button(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
        self.body = self.get_content_area()
        self.device = ""
        self.fstype = ""
        self.target = ""
        self.options = ""
        self.fstabmodel = fstabmodel
        self.entryobject = None
        self.fslabel = gtk.Label()
        self.fslabel.set_property('width-chars', 12)
        self.fslabel.set_property('xalign', 0.0)
        
        topbox = gtk.HBox()
        txt = "<b>%s</b>\n\n%s"% (
            _("Add new entry to /etc/fstab"),
            _("Select the partition and setup the options.") + "  " + \
            _("Default values work in most cases.") + " " + \
            _("This list only displays the partitions that are not already setup in /etc/fstab.")
            )
        topbox.pack_start(widgets.DLabel(txt), True, True, 4)
        self.body.pack_start(topbox, False, True, 4)
        
        intbox0 = gtk.HBox()
        lbPartSel = gtk.Label(_("Select Partition"))
        cbPartSel = gtk.combo_box_new_text()
        cbPartSel.set_active(0)
        intbox0.pack_start(lbPartSel, False, False, 4)
        intbox0.pack_start(cbPartSel, False, True, 2)
        #intbox1 = gtk.HBox()
        fslabel = gtk.Label()
        fslabel.set_use_markup(True)
        fslabel.set_label("<b>%s</b>"% _("Filesystem:"))
        intbox0.pack_start(fslabel, False, False, 2)
        intbox0.pack_start(self.fslabel, False, False, 2)
        #intbox1.pack_start(fslabel, False, False, 4)
        #intbox1.pack_start(self.fslabel, False, True, 2)
        intbox2 = gtk.HBox()
        targetlabel = gtk.Label()
        targetlabel.set_use_markup(True)
        targetlabel.set_label("<b>%s</b>"% _("Mount point"))
        self.targetbox = gtk.Entry()
        self.targetbox.connect('changed', self.targetbox_change_event)
        self.btSelectTarget = widgets.MyStockButton("Select",gtk.STOCK_OPEN)
        intbox2.pack_start(targetlabel, False, False, 4)
        intbox2.pack_start(self.targetbox, True, True, 2)
        intbox2.pack_start(self.btSelectTarget, False, False, 2)
        intbox3 = gtk.HBox()
        optionslabel = gtk.Label()
        optionslabel.set_markup("<b>%s</b>"% _("Mount Options"))
        self.optionsbox = gtk.Entry()
        intbox3.pack_start(optionslabel, False, False, 4)
        intbox3.pack_start(self.optionsbox, True, True, 2)
        self.btSelectTarget.connect("clicked", self.select_target_event)
        cbPartSel.connect('changed', self.combobox_change_event)
        intbox4 = gtk.HBox()
        mountcb = gtk.CheckButton(_("Mount partition immediately"))
        mountcb.connect('toggled', self.mountcb_toggle_event)
        mountcb.set_active(True)
        intbox4.pack_start(mountcb, False, True, 4)
        
        self.body.pack_start(intbox0, False, True, 12)
        #self.body.pack_start(intbox1, False, True, 2)
        self.body.pack_start(intbox2, False, True, 2)
        self.body.pack_start(intbox3, False, True, 2)
        self.body.pack_start(intbox4, False, True, 2)
        
        # Load all partitions to the combobox.
        self.load_available_partitions(cbPartSel)
        
        # Set the first item in the list visible
        cbPartSel.set_active(0)
        
        self.set_size_request(450, 300)
        self.body.show_all()

    def mountcb_toggle_event(self, widget=None):
        if not self.entryobject:
            return
        self.entryobject._mount_now = widget.get_active()

    def targetbox_change_event(self, widget=None):
        self.entryobject.mountpoint = widget.get_text()
        return
    
    def load_available_partitions(self, widget):
        """ Add the available partitions to widget.  Widget is a combobox"""
        allpartitions = FSTAB.list_all_system_partitions()
        for item in allpartitions:
            if not self.fstabmodel.hasEntry(item):
                widget.append_text(item)
    
    def select_target_event(self, widget=None):
        """ Browse to select the target directory """
        dia = gtk.FileChooserDialog(
            parent=self,
            title=_("Select mountpoint"),
            buttons=(gtk.STOCK_OK, gtk.RESPONSE_OK,
            gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL))
        dia.set_action(gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER)
        dia.set_current_folder("/media")
        
        res = dia.run()
        dia.hide()
        if res == gtk.RESPONSE_OK:
            self.targetbox.set_text(dia.get_filename())
            self.entryobject.mountpoint = dia.get_filename()
        dia.destroy()

    def combobox_change_event(self, combobox):
        sel = combobox.get_active_text()
        self.entryobject = FSTAB.FstabEntry(sel)
        self.entryobject.filesystem = self.entryobject.find_filesystem()
        self.entryobject.options = self.entryobject.find_default_options()
        # set the filesystem text on a label for informational puposes.
        self.fslabel.set_label(self.entryobject.filesystem)
        # set the default options
        self.optionsbox.set_text(self.entryobject.options)
        self.device = sel
        if "swap" in self.entryobject.filesystem:
            self.targetbox.set_text("none")
            self.targetbox.set_property("sensitive", False)
            self.btSelectTarget.set_property("sensitive", False)
        else:
            self.targetbox.set_property("sensitive", True)
            self.btSelectTarget.set_property("sensitive", True)
            defmpoint = self.entryobject.device.replace('/dev/','/mnt/')
            self.targetbox.set_text(defmpoint)
    
    def do_add_entry(self):
        """ Add an entry to fstab.  This method needs to check to make sure
        all parameters are right to call add_entry on the FSTAB.FstabModel"""
        if self.fstabmodel.hasEntry(self.entryobject.device):
            return dialogs.error(
                _("An entry for this device already exists.  Please delete it and add it again.")
                )
        try:
            self.fstabmodel.addEntry(self.entryobject)
            logger.debug("Added %s to /etc/fstab"% self.entryobject.device)
        except AssertionError as e:
            logger.error("Unable to add %s to %s"% (self.entryobject.device, e))
            return dialogs.error(
                _("Unable to add entry.") + " \n\n" + e)
        # Create the mountpoint if it does not exist
        if not os.path.exists(self.entryobject.mountpoint):
            os.makedirs(self.entryobject.mountpoint)
        # Mount the partition if the user so chose to
        if self.entryobject._mount_now:
            self.entryobject.mount()
        return

    
    
