#!/usr/bin/env python

#    This file is part of VASM.
#
#    VASM is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License v3 as published by
#    the Free Software Foundation.
#
#    VASM is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VASM.  If not, see <http://www.gnu.org/licenses/>.

__author__ = "Moises Henriquez"
__author_email__ = "moc.liamg@xnl.E0M"[::-1]

""" Module to provide a graphical interface for enabling / disabling software sources """

import gtk
import sys
import os
from vasm.backend import REPOS
from vasm.ui.gtk2.support import dialogs, widgets
from vasm.backend.utils import _
import logging

logger = logging.getLogger('vasm')

# vasm flags
VASM_CATEGORY = _("System")
VASM_LABEL=_("Software Sources")
VASM_ICON=gtk.STOCK_NETWORK


def __vasm_test__():
    """ Set of tests to decide when to display this module
    """
    if not os.path.exists("/etc/slapt-get/slapt-getrc"):
        return False
    return os.geteuid() == 0

class RepoManager(widgets.VasmModule):
    def __init__(self, parent):
        self._parent = parent
        tx = _("Select and prioritize your sources for system software updates.")
        widgets.VasmModule.__init__(self, parent, VASM_LABEL, tx)
        
        self.datamodel = REPOS.SlaptgetRcModel()
        self.datamodel.add_observer(self.refresh_dataview)
        
        sw = gtk.ScrolledWindow()
        sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        
        self.treeview = self.get_list_treeview()
        treebox = gtk.HBox()
        sw.add(self.treeview)
        treebox.pack_start(sw, True, True, 4)
        
        # get the current list and append it to the tree
        self.datamodel.notify()
        
        self.pack_start(treebox, True, True, 4)
        
        # add buttons to the action area
        btapply = gtk.Button(stock=gtk.STOCK_APPLY)
        btclose = gtk.Button(stock=gtk.STOCK_CANCEL)
        btapply.connect('clicked', self.apply_event, self.treeview.get_model())
        btclose.connect('clicked', self.close_module)
        self.add_button(btclose)
        self.add_button(btapply)

    def apply_event(self, widget, treemodel):
        """ Apply the changes in treemodel to self.datamodel """
        changes = []
        for item in treemodel:
            changes.append((item[0], item[2], item[3]))
        try:
            self.datamodel.save_changes(changes)
            logger.debug("Updated /etc/slapt-get/slapt-getrc")
        except IOError, (errorno, errormsg):
            logger.error("Unable to save changes to /etc/slapt-get/slapt-getrc. %s"%errormsg + \
            " A copy of your original slapt-getrc file was saved to /etc/slapt-get/slapt-getrc.bak")
            return dialogs.error(
                _("Error while saving changes.  See /var/log/vasm.log"))
        return dialogs.info(_("Your changes were saved to /etc/slapt-get/slapt-getrc"))
    
    def priority_edit_event(self,combo, path, text, treemodel):
        """ Set what happens when the priority on a repo is changed"""
        if treemodel[path][0] is False:
            return dialogs.warning(
                _("This repository is disabled.  Enable it before changing it's priority value.")
            )
        # Disable this priority on all other enabled repos.
        for item in treemodel:
            if item[0] is True:
                if item[-1] == text:
                    item[-1] = "Default"
        # Set the new changed priority on the selected repo
        treemodel[path][-1] = text
    
    def repo_toggle_event(self, cell, path, model, column):
        item = model.get_iter(path)
        model.set_value(item, column, not cell.get_active())

    def refresh_dataview(self):
        """ Refresh the treeview showing the contents of the slapt-getrc file"""
        store = self.treeview.get_model()
        store.clear()
        for repo, priority in self.datamodel.get_current_active_repos():
            store.append([True, True, repo, priority])
        # now list the disabled sources
        for repo, priority in self.datamodel.get_current_disabled_repos():
            for item in store:
                if item[0] is True and item[-1] == priority:
                    priority = "Default"
            store.append([False, True, repo, priority])

    def get_list_treeview(self):
        """Create a treeview to display the data """
        treestore = gtk.ListStore(bool, bool, str, str)
        priostore = gtk.ListStore(str)
        priostore.append(["Default"])
        priostore.append(["Official"])
        priostore.append(["Preferred"])
        crTgl = gtk.CellRendererToggle()
        crTgl.connect("toggled", self.repo_toggle_event, treestore, 0)
        crRepo = gtk.CellRendererText()
        crPrio = gtk.CellRendererCombo()
        crPrio.connect('edited', self.priority_edit_event, treestore)
        crPrio.set_property('model', priostore)
        crPrio.set_property('text-column', 0)
        crPrio.set_property('has-entry', False)
        crPrio.set_property('editable', True)
        
        colTgl = gtk.TreeViewColumn(_("Enabled"), crTgl, active=0, activatable=1)
        colRepo = gtk.TreeViewColumn(_("Repo URL"), crRepo, text=2)
        colPrio = gtk.TreeViewColumn(_("Priority"), crPrio, text=3)
        
        tree = gtk.TreeView(treestore)
        for col in (colTgl, colRepo, colPrio):
            tree.append_column(col)
            col.set_resizable(True)
        
        return tree

def __run__(parent):
    plugin = RepoManager(parent)
    return plugin.launch()






