#!/usr/bin/env python

#    This file is part of VASM.
#
#    VASM is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License v3 as published by
#    the Free Software Foundation.
#
#    VASM is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VASM.  If not, see <http://www.gnu.org/licenses/>.

""" tui_autologin.py
Text mode module to enable and set automatic login parameters.  GDM must be the
login manager in use to for this to work.

"""
__author__ = "Moises Henriquez"
__author_email__ = "moc.liamg@xnl.E0M"[::-1]

import urwid
import sys
import os
from vasm.backend import LOGINMANAGERS, USERADMIN, GDMHELPER
from vasm.ui.tui.support import dialogs, widgets
from vasm.backend.utils import _
import logging

logger = logging.getLogger('vasm')

VASM_CATEGORY = "Graphical Environment"
VASM_LABEL = "Automatic Login Settings"

class GdmSetup(dialogs.ModuleDialog):
    def __init__(self, parent):
        self.parent = parent
        self._widgets = []
        radios = []
        self.settingsmodel = GDMHELPER.SettingsModel()
        self._current_state = self.settingsmodel.get_current_autologin_setting()
        self._current_aluser = self.settingsmodel.get_current_autologin_user()
        rbNoLogin = urwid.RadioButton(radios,
                                      _("Disable automatic login."),
                                      user_data = 'false',
                                      state = self._current_state == 'false',
                                      on_state_change = self.autologin_toggle_event)
        noexp = urwid.Text(" "* 4 + _("Users must enter login name and password."))
        rbAutoLogin = urwid.RadioButton(radios,
                                        _("Enable automatic login."),
                                        user_data = 'true',
                                        state = self._current_state == 'true',
                                        on_state_change = self.autologin_toggle_event)
        yesexp = urwid.Text(" "*4 + _("The following user will be automatically logged in."))
        
        self._widgets.extend([urwid.AttrWrap(rbNoLogin, 'button'),
                     noexp,
                     urwid.AttrWrap(rbAutoLogin, 'button'),
                    yesexp])
        self._logins_picker = self._get_users_list()
        self._widgets.append(self._logins_picker)
        dialogs.ModuleDialog.__init__(
            self,
            header=_("Automatic Login Settings"),
            body=self._widgets,
            desc = [_("Select how users log into the graphical desktop."),"\n"],
            parent = self.parent,
            buttons = [
                ("Save", self.save_settings),
                ("Back", self.close_module)]
        )
        
        self.set_body(urwid.LineBox(self.get_body()))
        self._autologin_enabled = self._current_state
        self._autologin_user = self._current_aluser or 'root'

        # move the focus to the body of the module
        self.set_focus('body')
    
    def save_settings(self, caller_widget=None):
        """ Make the call to save the settings """
        xdm_model = LOGINMANAGERS.LoginManagers()
        # check if gdm is the current login manager.
        if xdm_model.get_current() != "GDM":
            # check if gdm is
            if "GDM" in xdm_model.list_existing():
                # force gdm
                xdm_model.set_manager("GDM")
            else:
                # throw a dialog because there is no gdm.
                dia = dialogs.Error(
                    parent = self.parent,
                    message = _("The GDM login manger is not found on this system.") + "\n" + \
                    _("These settings can only be used with the gdm login manager.") + "\n" + \
                    _("Please install gdm and try again."),
                    buttons = [("OK", self.close_module)]
                )
                return self.pop_dialog(dia)

        try:
            self.settingsmodel.save_settings(
                str(self._autologin_enabled),
                str(self._autologin_user)
                )
        except IOError as e:
            dia = dialogs.Error(
                parent = self.parent,
                message = _("There was an error saving the your automatic login preferences.") + "\n" + e,
                buttons = [("OK", self.close_module)]
                )
            logger.error("Error saving automatic login settings: %s"% e)
            return self.pop_dialog(dia)
        # Display a success dialog
        dia = dialogs.Info(
            parent = self.parent,
            message = _("Your settings have been successfully saved.") + "\n" + \
            _("Your changes will take effect after the next system reboot."),
            buttons = [("OK", self.close_module)]
            )
        logger.debug("Saved automatic login settings.")
        return self.pop_dialog(dia)
    
    def autologin_toggle_event(self, radio, newstate, user_data):
        """ user_data has the corrent setting to be parsed"""
        if newstate is True:
            self._autologin_enabled = user_data

    def autologin_user_toggle_event(self, radio, newstate, user_data):
        """ user_data contains the login name of the selected radio button """
        if newstate is True:
            self._autologin_user = user_data
    
    def _get_users_list(self):
        allusers = USERADMIN.UserModel().listUsers()
        logins = []
        _names = []
        for account in allusers:
            rb = urwid.RadioButton(_names,
                                   account.login,
                                   state = self._current_aluser == account.login,
                                   user_data = account.login,
                on_state_change = self.autologin_user_toggle_event)
            w = urwid.AttrWrap(rb, 'button')
            logins.append(
                urwid.Columns([('fixed', 6, urwid.Text(" ")), w], 0))
        box = urwid.Pile(logins)
        #box = urwid.ListBox(logins)
        return box
    


def __vasm_test__():
    return os.geteuid() == 0


def __run__(parent):
    mod = GdmSetup(parent)
    parent.pop_module(mod)
