#!/usr/bin/env python

#    This file is part of VASM.
#
#    VASM is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License v3 as published by
#    the Free Software Foundation.
#
#    VASM is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VASM.  If not, see <http://www.gnu.org/licenses/>.

""" tui_service.py
text mode module to interact with system services
"""
__author__ = "Moises Henriquez"
__author_email__ = "moc.liamg@xnl.E0M"[::-1]

import urwid
import os
from vasm.ui.tui.support import dialogs, widgets
from vasm.backend.utils import _, datapath
from vasm.backend import SERVICES
import logging

logger = logging.getLogger('vasm')

VASM_CATEGORY = "System"
VASM_LABEL = "System Services Configuration"

class ModDialog(dialogs.ModuleDialog):
    """ Subclass ModuleDialog to fix the esc, q and b key stroke calls """
    def __init__(self, *args, **kwargs):
        dialogs.ModuleDialog.__init__(self, *args, **kwargs)
        
    def keypress(self, size, key):
        if key in ("esc", "q", "b"):
           return self._trigger_cancel_method()
        super(ModDialog, self).keypress(size, key)

class ServicesModule(ModDialog):
    """ Interact with system services """
    def __init__(self, parent):
        self.parent = parent
        self._title_text = _("System Services Configuration")
        desc = _("Select which service to enable for each runlevel.  Press 'OK' to save changes")
        _body = []
        # create a servicemodel object
        self.servicemodel = SERVICES.ServiceModel()
        # get the list of service. This comes in a list of lists like this
        # |service| garbage_bool| Init2 value | init 3 value| init 4 value| init 5 value
        # The 2nd item in each list is useless for the tui interface.
        svcs = self.servicemodel.list_services_with_status()
        
        # we create this dictionary to store the default data read from the
        # backend module.  Each item is composed as follows.
        # "service":{2:bool, 3:bool, 4:bool, 5:bool}
        self._service_settings = {}
        rows = []
        row_hdrs = [urwid.AttrWrap(w, 'column header') for w in [urwid.Text(t) for t in (
            "Service",
            "Runlevel 2",
            "Runlevel 3",
            "Runlevel 4",
            "Runlevel 5"
        )]]
        #_body.append(urwid.Columns(row_hdrs))
        lines = []
        for line in svcs:
            name, trash, init2, init3, init4, init5 = line
            # Add the data to the dictionary
            self._service_settings[name]={2:init2, 3:init3, 4:init4, 5:init5}
            _line = [
                urwid.AttrWrap(urwid.Text(name), 'button'),
                urwid.AttrWrap(widgets.SensibleCheckBox(
                    "",state=init2, on_state_change = self.on_checkbox_toggle,
                    on_focus=self.show_service_description,
                    user_data=(name, 2)), 'button', 'button sel'),
                urwid.AttrWrap(widgets.SensibleCheckBox(
                    "",state=init3, on_state_change = self.on_checkbox_toggle,
                    on_focus=self.show_service_description,
                    user_data=(name, 3)), 'button', 'button sel'),
                urwid.AttrWrap(widgets.SensibleCheckBox(
                    "",state=init4, on_state_change = self.on_checkbox_toggle,
                    on_focus = self.show_service_description,
                    user_data=(name, 4)), 'button', 'button sel'),
                urwid.AttrWrap(widgets.SensibleCheckBox(
                    "",state=init5, on_state_change = self.on_checkbox_toggle,
                    on_focus = self.show_service_description,
                    user_data=(name, 5)), 'button', 'button sel')
            ]
            lines.append(_line)

        for line in lines:
            _body.append(urwid.Columns(line, dividechars=5))
        
        buttons = [("Help", self.show_help),("OK", self.ok_button_event),
            ("Cancel", self.parent.return_home),
            ]
        
        ModDialog.__init__(
            self,
            parent = self.parent,
            body = _body,
            header = self._title_text,
            desc = [desc,urwid.Columns(row_hdrs)],
            buttons = buttons
        )
        
        self.body = urwid.LineBox(self.get_body())
        self.set_focus('body')
    
    def show_help(self, widget=None):
        """ Pop a dialog to display some helpful text about system services and runlevels"""
        hfpath = os.path.join(datapath, 'servicehelp.txt')    
        hfile = open(hfpath, 'r')
        data = hfile.readlines()
        hfile.close()
        
        dia = dialogs.Info(
            parent = self.parent,
            message = "".join(data),
            buttons = [("OK", self.return_to_module)]
        )
        return self.parent.pop_file_dialog(dia)
    
    def show_service_description(self, widget):
        name, init = widget.user_data
        statustxt = SERVICES.get_service_desc(name)
        self.statusbar.set_text('%s: %s'% (name, statustxt))
    
    def on_checkbox_toggle(self, widget, newvalue, data):
        """ data contains a tuple with (service_name, init_level) """
        # We modify the data in the dictionary: self._service_settings
        # it would look like this
        #
        # lst = {"sshd":{2:False, 3:True, 4:False, 5:True}}
        name, init = data
        self._service_settings[name][init] = newvalue

    def save_service_config_changes(self, widget=None):
        """ Save the changes to the system services configuration"""
        # access the main loop and stop the screen while we perform changes
        master = self.parent._mainloop
        master.screen.stop()
        errors = False
        svcnames = self._service_settings.keys()
        for svc in svcnames:
            for rl in range(2, 6):
                if self._service_settings[svc][rl] is True:
                    logging.debug('Enabling service %s for runlevel %s'% (svc, rl))
                    code, output = self.servicemodel.enable_service(svc, rl)
                    print output.strip()
                    if code > 0:
                        errors = True
                else:
                    logging.debug('Disabling service %s from runlevel %s'% (svc, rl))
                    code, output = self.servicemodel.disable_service(svc, rl)
                    print output.strip()
                    if code > 0:
                        errors = True
        # Start the screen again
        master.screen.start()
        if errors:
            return self.show_error_dialog()
        return self.show_success_dialog()
    
    def show_error_dialog(self, widget=None):
        """ Display a dialog to indicate errors """
        dia = dialogs.Error(
            parent = self.parent,
            message = _(
                "Some errors were encountered while setting service settings."),
            buttons = [("OK", self.parent.return_home)]
        )
        return self.parent.pop_module_dialog(dia)
    
    def show_success_dialog(self, widget=None):
        """ Display a success dialog """
        dia = dialogs.Info(
            parent = self.parent,
            message = _("All changes have been successfully applied."),
            buttons = [("OK", self.parent.return_to_module)]
        )
        return self.parent.pop_module_dialog(dia)
    
    def pop_dialog(self, dia):
        """ pass a pop-up dialog back to the parent """
        return self.parent.pop_module_dialog(dia)        
    
    def ok_button_event(self, widget):
        """ Save the changes as specified in the system """
        dia = dialogs.Question(
            parent=self.parent,
            message = _("Are you sure you want to save changes now?"),
            buttons = [
                ("Yes", self.save_service_config_changes),
                ("No", self.parent.return_to_module_stack)
            ]
        )
        return self.parent.pop_module_dialog(dia)
    
def __vasm_test__():
    """ Set of tests to determine who sees this module and when """
    return os.geteuid() == 0

def __run__(parent):
    """ Define what to do when this module is selected from the launcher """
    wid = ServicesModule(parent)
    parent.pop_module(wid)

 