#!/usr/bin/env python

#    This file is part of VASM.
#
#    VASM is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License v3 as published by
#    the Free Software Foundation.
#
#    VASM is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VASM.  If not, see <http://www.gnu.org/licenses/>.

""" tui_usermanage.py
    Text UI module for user management. """

__author__ = "Moises Henriquez"
__author_email__ = "moc.liamg@xnl.E0M"[::-1]

import urwid
import os
from vasm.backend.utils import _
from vasm.backend import PASSWORDS, USERADMIN
from vasm.ui.tui.support import dialogs, widgets
import logging

logger = logging.getLogger('vasm')

VASM_CATEGORY = "System"
VASM_LABEL = "Users and Passwords"

class GroupsPrompt(urwid.Frame):
    """ Dialog to set the users groups """
    def __init__(self, parent=None, account=None):
        self.parent = parent
        self.topparent = parent.parent
        self._user_groups = []
        self._buttons = [("Apply", self.do_modify_groups),
            ("Cancel", self.parent.return_to_module)]
        self.buttons = []
        for but, cb in self._buttons:
            self.buttons.append(
                urwid.AttrWrap(urwid.Button(but, cb), 'button', 'button sel'))
        bar = urwid.GridFlow(self.buttons, 15, 1, 1, 'center')
        title = urwid.Text(_("USER GROUP MEMBERSHIPS"), 'right')
        #self._get_user_groups()
        self.account = account
        _content = [
            ('flow', widgets.blank),
            self._get_user_groups(),
            ('flow', widgets.blank),
        ]
        urwid.Frame.__init__(self,
                             header = urwid.AttrWrap(urwid.LineBox(title), 'body'),
                             body = urwid.Pile(_content),
                             footer = urwid.AttrWrap(bar, 'body'))

    def keypress(self, size, key):
        """ react to 'q' or 'esc' key strokes """
        if key in ('q', 'esc'):
            if len(self._buttons) == 1:
                return self._buttons[0][1]()
            else:
                for bt, cb in self._buttons:
                    if bt in ("Cancel", "cancel", "NO", "no", "No", "Back", "back", "Done", "done"):
                        return cb()
        elif key == 'tab':
            if self.focus_part == 'body':
                self.set_focus('footer')
            else:
                self.set_focus('body')
        else:
            return super(GroupsPrompt, self).keypress(size, key)
    
    def do_modify_groups(self, widget):
        """ Apply the changes """
        cgroups = self._user_groups
        # store the changes to the account
        self.account.groups = cgroups
        try:
            self.account.save_groups_changes()
            logger.info('Group membership for %s changed'% self.account.login)
        except USERADMIN.AccountError as e:
            dia = dialogs.Error(
                parent = self.parent,
                message = _("The following error was encountered while saving changes") + ":\n\n%s"% e,
                buttons = [("OK", self.parent.return_to_module)]
                )
            logger.error("Error modifying group memberships for %s"% (self.account.login,e))
            return self.parent.pop_dialog(dia, 3)
        dia = dialogs.Info(
            parent = self.parent,
            message = _("Your changes have been successfully applied to this account."),
            buttons = [("OK", self.parent.return_to_module)]
            )
        return self.parent.pop_dialog(dia, 3)
    
    def _get_user_groups(self):
        """ Return a listbox with checkboxes for all the groups this user belongs to """
        content = []
        allgroups = []
        for grp in USERADMIN.__distro_groups__:
            allgroups.append((grp, USERADMIN.__distro_groups__[grp][0]))
        # produce a list of all the groups this user is already a member of
        memberships = self.account.groups
        # Loop through all groups and check if the user is already a member.
        
        for grp, desc in allgroups:
            state = grp in memberships
            if state is True:
                if not grp in self._user_groups:
                    self._user_groups.append(grp)
            cb = urwid.CheckBox("[%s] %s"% (grp, desc), state=state,
                                on_state_change=self.toggle_group_membership, user_data=grp)
            wid = urwid.AttrWrap(cb, 'dialog body')
            content.append(wid)
        return urwid.ListBox(content)
    
    def toggle_group_membership(self, widget, newstate, user_data):
        """ Toggle event for the checkboxes.  user_data contains the group name"""
        if newstate is True:
            if not user_data in self._user_groups:
                self._user_groups.append(user_data)
        else:
            if user_data in self._user_groups:
                self._user_groups.remove(user_data)

class PasswordPrompt(dialogs.ModuleDialog):
    """ Dialog pops up to ask for new password """
    def do_change_password(self, widget):
        """ Perform some tests, and then change the password """
        pass1 = self.passbox1.get_edit_text()
        pass2 = self.passbox2.get_edit_text()
        
        # test if passwords match
        if not PASSWORDS.test_passwords_match(pass1, pass2):
            #error out
            dia = dialogs.Error(
                parent = self.parent,
                message = _("The passwords you entered do not match.  Please enter the same password twice."),
                buttons = [("OK", self.parent.return_to_module_stack)]               
            )
            # pop a 3rd level dialog
            return self.parent.pop_module_dialog(dia, 3)
        # This test is different because root can set the password to anything,
        # but an empty password will cause problems.
        if self.passbox2.get_edit_text() == "": #PASSWORDS.test_password_length(pass2):
            # password is too short
            dia = dialogs.Error(
                parent = self.parent,
                message = _("The password cannot be blank.  A password that is at least 6 characters long is recomended."),
                buttons = [("OK", self.parent.return_to_module_stack)]
            )
            # pop the dialog
            return self.parent.pop_module_dialog(dia, 3)        
        # After all tests passed, make the call to change the password.

        try:
            # root doesn't need to know the old password.
            ret = PASSWORDS.change_password(user=self.login,
                                            oldpassword="foo",
                                            newpassword = pass2)
            logger.info('Password for %s was changed'% self.login)
        except:
            ret = 1
            dia = dialogs.Error(
                parent = self.parent,
                message = _("Pexpect error"),
                buttons = [("OK", self.parent.return_to_module_stack)]
            )
            logger.error('Pexpect returned error while changing password.')
        if ret > 0:
            dia = dialogs.Error(
                parent = self.parent,
                message = _("Error changing password.  The password was not changed.") + "\n" + \
                _("Make sure your password is at least 6 characters long"),
                buttons = [("OK", self.parent.return_to_module_stack)]
            )
        else:
            dia = dialogs.Info(
                parent = self.parent,
                message = "%s %s"% (_("Password has been successfully changed for"), self.login),
                buttons = [("OK", self.parent.return_to_module)]                
            )
        
        # blank out the form
        self.clear_all_input()
        
        # pop the last dialog
        return self.parent.pop_module_dialog(dia, 3)
    
    def clear_all_input(self):
        """ Clear all the data that was entered into the form """
        [ w.set_edit_text("") for w in (self.passbox1, self.passbox2)]
        
    def __init__(self, parent=None, login=""):
        """ A pop-up dialog to ask user for passwords input """
        self.parent = parent
        buttons = []
        self.login = login
        self.buttons = []
        
        # add the ok buttons
        self._buttons = [("OK", self.do_change_password),
            ("Cancel", self.parent.return_to_module_stack)]
        for but, cb in self._buttons:
            self.buttons.append(
                urwid.AttrWrap(urwid.Button(but, cb), 'button', 'button sel'))
        bar = urwid.GridFlow(self.buttons, 15, 1, 1, 'center')
        
        self.currpassbox = urwid.Edit(_("Current Password: "), mask="*")
        self.passbox1 = urwid.Edit(_("Enter New Password: "), mask="*")
        self.passbox2 = urwid.Edit(_("Verify New Password: "), mask="*")
        
        _header = urwid.Text(" ")
        
        # root does not need to know the current passwords.
        if os.geteuid() > 0:
            _header.set_text(_("CHANGE MY PASSWORD"), align='right')
            #title = _("CHANGE MY PASSWORD")
            message=_("Change your password by filling in the fields below.  " \
                      "Select a password that is at least 6 characters long and " \
                      "Press 'OK' when you are done.")
            _content = [
                widgets.blank,
                self.currpassbox,
                self.passbox1,
                self.passbox2,
                widgets.blank]

        else:
            if login == 'root':
                _header.set_text(_("CHANGE ROOT PASSWORD"))
                _header.set_align_mode('right')
                #title = _("CHANGE ROOT PASSWORD")
                message=_("This will change the system's root password.  "\
                          "This is the system administrators account.")
            else:
                _header.set_text(_("CHANGE USERS PASSWORD"))
                _header.set_align_mode('right')
                #title = _("CHANGE USERS PASSWORD")
                message=_("You are about to change the password for ") + login + \
                ".  " + \
                _("Enter the new password in the fields below and press 'OK' when finished.")
            _content = [
                        widgets.blank,
                        self.passbox1,
                        self.passbox2,
                        widgets.blank]
        dialogs.ModuleDialog.__init__(
            self,
            parent = self.parent,
            header = _header.text,
            desc = [message],
            body = _content,
            buttons = self._buttons            
        )
        self.set_focus('body')

    def _get_action_area(self):
        buttons = []
        for but, cb in self.buttons:
            buttons.append(urwid.AttrWrap(
                    urwid.Button(but, cb), 'button', 'button sel'
                )
            )
        bar = urwid.GridFlow(buttons, 15, 1, 1, 'center')
        return bar

    def _get_footer(self):
        """ Return a pile that can be  used for the window's footer"""
        buttons = self._get_action_area()
        return buttons
        #statusline = [self.statusbar, buttons]
        #return urwid.Columns(statusline)

    def keypress(self, size, key):
        """ react to 'q' or 'esc' key strokes """
        if key in ('q', 'esc'):
            if len(self._buttons) == 1:
                return self._buttons[0][1]()
            else:
                for bt, cb in self._buttons:
                    if bt in ("Cancel", "cancel", "NO", "no", "No", "Back", "back", "Done", "done"):
                        return cb()
        elif key == 'tab':
            if self.focus_part == 'body':
                self.set_focus('footer')
            else:
                self.set_focus('body')
        else:
            return super(PasswordPrompt, self).keypress(size, key)


class UserAddForm(urwid.ListBox):
    """ The form that contains all the fields to be filled for the user add
    side form.  This widget shall only be used as a right pane to UserWindow
    
    arguments:
        parent  - Parent widget (should be a subclass of UserWindow)
        buttons - list of (button, callback) 
    """
    def __init__(self, parent=None, usermodel=None):
        self.parent = parent
        self._parent = self.parent.parent
        hdt = "%s.  %s.  %s.  %s"% (
            _("Use UP/DOWN to browse through existing users and see account details and maintenance options"),
            _("Fill in the fields below and click 'Create Account' to create a new account"),
            _("Fields marked with a '*' are required."),
            _("Press 'esc' to cancel.")
        )
        self.usermodel = usermodel
        self.privileged_account = False
        act_types = []
        hdr = urwid.Text(hdt)
        _list = [hdr]
        self.loginbox = urwid.Edit('Login Name* : ')
        self.rnamebox = urwid.Edit('Real Name : ')
        self.passbox1 = urwid.Edit('Enter Passsword* : ', mask="*")
        self.passbox2 = urwid.Edit('Verify Password* : ', mask="*")
        _list.append(widgets.blank)
        for w in (
            self.loginbox, self.rnamebox, self.passbox1, self.passbox2,
        ):
            _list.append(urwid.AttrWrap(w, 'button'))
        _list.append(widgets.blank)
        _list.append(urwid.Text('Account Type*:'))
        _list.append(urwid.AttrWrap(
            urwid.RadioButton(act_types, _('Unprivileged User'),
                              on_state_change=self.account_type_toggle,
                              user_data=False), 'button'))
        _list.append(urwid.AttrWrap(
            urwid.RadioButton(act_types, _('Administrator'),
                              on_state_change=self.account_type_toggle,
                              user_data=True), 'button'))
        _list.append(widgets.blank)
        
        # create buttons for creating account or cancelling action
        btCreate = urwid.Button(_("Create Account"), on_press = self.create_account)
        btCancel = urwid.Button(_("Cancel"), on_press=self.cancel_new_user)
        
        _list.append(urwid.GridFlow([urwid.LineBox(urwid.AttrWrap(btCreate,'button', 'button sel')),
                                     urwid.LineBox(urwid.AttrWrap(btCancel, 'button', 'button sel'))
                                     ], 25, 1, 1, 'left'))

        urwid.ListBox.__init__(self, _list)
    
    def account_type_toggle(self, toggle, newstate, privileged=False):
        """ Set the account type when the radio button is toggled.
        the privileged argument is the user_data value on the radio button.  """
        if newstate is True:
            self.privileged_account = privileged
    
    def _emulate_left_arrow(self, times=1):
        ml = self.parent.parent._mainloop
        for x in range(0, times):
            ml.process_input(['left'])
            self.keypress((10, 2), 'left')
    
    def cancel_new_user(self, widget):
        """ Clear the form and return focus to the left panel """
        for entry in (
            self.loginbox, self.rnamebox, self.passbox1, self.passbox2):
            entry.set_edit_text("")
        # return focus to the left panel
        return self._emulate_left_arrow(2)
    
    def keypress(self, size, key):
        if key in ("esc",):
            return self._emulate_left_arrow()
        return super(UserAddForm, self).keypress(size, key)
        
    def test_account_parameters(self):
        """ Run some tests on the specified parameters to see if we can create an account with them.
        Returns True when all parameters are correct.  """
        # Check if we have a login name
        if not self.loginbox.get_edit_text():
            dia = dialogs.Error(
                parent = self._parent,
                message = _("Please enter a login name.  Every account needs a login name."),
                buttons = [("OK", self._parent.return_to_module_stack)])
            return self._parent.pop_module_dialog(dia)
        # test if the username is avialable.  This should return False if the login
        # is available, True if it is taken
        if self.usermodel.hasUser(self.loginbox.get_edit_text()):
            dia = dialogs.Error(
                parent = self._parent,
                message = _(
                    "The login name you entered already exists on this system.  Pick a unique login name."),
                buttons = [("OK", self._parent.return_to_module_stack)])
            return self._parent.pop_module_dialog(dia)
        # check for passwords matching
        pass1 = self.passbox1.get_edit_text()
        pass2 = self.passbox2.get_edit_text()
        if not PASSWORDS.test_passwords_match(pass1, pass2):
            dia = dialogs.Error(
                parent = self._parent,
                message = _("The passwords you specified do not match.  Please enter the same password twice."),
                buttons = [("OK", self._parent.return_to_module_stack)])
            return self._parent.pop_module_dialog(dia)
        if not PASSWORDS.test_password_length(pass2):
            dia = dialogs.Error(
                parent = self._parent,
                message = _("The password you entered is too short.  Enter a password that is at least 6 characters long."),
                buttons = [("OK", self._parent.return_to_module_stack)])
            return self._parent.pop_module_dialog(dia)
        # All settings are good... Let's create an account
        return True

    def create_account(self, widget=None):
        """ Create the user account (after the tests have passed) """
        if not self.test_account_parameters():
            return
        # get the information to create the account
        lname = self.loginbox.get_edit_text()
        rname = self.rnamebox.get_edit_text()
        passw = self.passbox2.get_edit_text()
        # cant set face icon for account in text mode, so just use a default.
        if os.path.exists("/usr/share/apps/kdm/pics/users/default1.png"):
            facepath = ("/usr/share/apps/kdm/pics/users/default1.png")
        else:
            facepath = None
        # get the groups based on the account type selected.
        reg_groups = []
        admin_groups = []
        for grp in USERADMIN.__distro_groups__:
            admin_groups.append(grp)
            if USERADMIN.__distro_groups__[grp][1] is True:
                reg_groups.append(grp)
        if self.privileged_account is True:
            groups = admin_groups
        else:
            groups = reg_groups
        
        # stop the screen to let the user know we are working.
        self.parent.set_status_text("Cearing user account")
        #        self._parent._mainloop.screen.stop()
        #        print _("Creating user account"),"..."
        # Pop a dialog to tell people what we're doing.
        #        dia = dialogs.Info(parent = self.parent,
        #                           message = _("Creating user account"),
        #                           buttons = [])
        #        self._parent.pop_module_dialog(dia, 2)
        # Now that we have the information, make the call to create the account.
        account = USERADMIN.User(new=True)
        account.login = lname
        if rname not in ("None", "", None):
            account.realname = rname
        else:
            account.realname = None
        account.realname = rname
        account.password = passw
        account.groups = groups
        account.faceicon = facepath
        try:
            self.usermodel.addUser(account)
            self.parent.set_status_text("")
            logger.info("Added new user %s"% account.login)
        except AssertionError as e:
            dia = dialogs.Error(
                parent = self.parent,
                message = _("The following error was encountered while adding the new account") + ": \n\n%s"% e,
                buttons = [("OK", self.parent.return_to_module)]
                )
            self.parent.set_status_text("")
            self.clear_form_values()
            logger.error("Unable to add account %s: %s"% (account.login, e))
            return self.parent.pop_dialog(dia)
        dia = dialogs.Info(
            parent = self.parent,
            message = _("User account was created successfully."),
            buttons = [("OK", self.parent.return_to_module)]
            )
        self.parent.set_status_text("")
        self.clear_form_values()
        return self.parent.pop_dialog(dia)
            
    def clear_form_values(self):
        """ Clear the data entered into the form """
        self.privileged_account = False
        self.loginbox.set_edit_text("")
        self.passbox1.set_edit_text("")
        self.passbox2.set_edit_text("")
        self.rnamebox.set_edit_text("")

class UserWindow(dialogs.ModuleDialog):
    """ urwid window that shows 2 panes.  On the left it shows user login names
        present in the system.  On the right, it shows details about that user
        and a button to change the user password or delete the account."""

    def __init__(self, parent):
        self.parent = parent
        self.current_user = None
        header = _("System users administration")
        _left_header = urwid.AttrWrap(urwid.Text(_("User")), 'column header')
        _right_header = urwid.AttrWrap(urwid.Text(
                                       " " + _("Account info / options")), 'column header')
        _top_cols = urwid.Columns(
            widget_list = [('fixed', 20, _left_header), _right_header], dividechars=1)  
        _body = []
        buttons = [("Back", self.parent.return_home)]
        # USERADMIN has a UserModel that we can use
        self.usermodel = USERADMIN.UserModel()
        # List the user accounts using CategoryLabel widgets.  This allows
        # to display user info when the user is selected.
        #        all = self.usermodel.list_all()
        self.usermodel.add_observer(self.refresh_user_accounts)
        _user_labels = []
        col0 = []
        #  Add a button to create new user account.
        self._nacct = widgets.CategoryLabel(
            _("New User"),
              focus_method = self.display_idle_message,
              enter_sensitive=True)
        self._nacct._select_method = self.switch_to_newuser_form
        self.leftpane = urwid.ListBox([])
        self.rightpane = widgets.ModuleRightPane(parent=self.parent, body=[])
        #self.rightpane = urwid.ListBox([])
        self.columns = urwid.Columns([('fixed', 20, self.leftpane), urwid.LineBox(self.rightpane)], 1)
        # List user accounts
        self.usermodel.notify()
        _body = urwid.Pile(
            widget_list=[
                ('flow', widgets.blank),
                self.columns,
                ])

        dialogs.ModuleDialog.__init__(
            self,
            parent = self.parent,
            body = _body, #_body,
            header = header,
            desc = [_top_cols],
            buttons = buttons
        )
        self.set_focus('body')
    
    def keypress(self, size, key):
        """ Handle esc keystroke correctly """
        if key == 'esc':
            focus = self.columns.focus_col
            if focus == 1:
                return self.get_body().keypress(size, key)
            else:
                return self._trigger_cancel_method()
        return super(UserWindow, self).keypress(size, key)

    def refresh_user_accounts(self, model):
        """ Method to refresh user accounts """
        col0 = []
        # add the non-changing stuff
        col0.append(urwid.AttrWrap(self._nacct, None, 'reveal focus'))
        col0.append(widgets.blank)
        col0.append(widgets.blank)
        for acct in model.listUsers():
            user = acct.login
            label = widgets.CategoryLabel(
                user,
                focus_method = self.display_user_details,
                enter_sensitive=True)
            label._select_method = self.switch_to_account
            widget = urwid.AttrWrap(label, None, 'reveal focus')
            col0.append(widget)

        self.userlisting = widgets.CategoryDisplay(col0)
        self.leftpane = urwid.ListBox(self.userlisting)
        self.columns.widget_list[0] = self.leftpane

    def switch_to_account(self, widget):
        """ Switch to account details.  Simply trigger a 'right' key stroke """
        ml = self.parent._mainloop
        ml.process_input(['right'])
        widget.keypress((10, 2), 'right')
    
    def display_idle_message(self, widget=None):
        neww = UserAddForm(self, self.usermodel)
        self.columns.widget_list[-1] = urwid.LineBox(neww)
    
    def switch_to_newuser_form(self, widget=None):
        """ Display the form to create a new user."""
        # Force the focus to move to the right pane automatically
        ml = self.parent._mainloop
        ml.process_input(['right'])
        widget.keypress((10,2), 'right')
    
    def display_user_details(self, widget=None):
        """ Display details about this account on the right pane """
        self.current_user = widget.label
        account = self.usermodel.getUser(self.current_user)
        #account = USERADMIN.User(widget.label)
        btdel = urwid.AttrWrap(urwid.Button(_("Delete User"),
                                            self.confirm_delete_account),'button', 'button sel')
        btperm = urwid.AttrWrap(urwid.Button(_("User Permissions"),
                                             self.prompt_user_groups), 'button', 'button sel')
        btpass = urwid.AttrWrap(urwid.Button(_("Change User Password"),
                                             self.prompt_password_change), 'button', 'button sel')
        if account.fullname in ("", "None", None):
            fn = _("Not Available")
        else:
            fn = account.fullname
        _widgets = [
            urwid.Text("%s : %s"% (_("Login Name"), account.login)),
            urwid.Text("%s : %s"% (_("Real Name"), fn)),
            urwid.Text("%s : %s"% (_("User ID"), account.uid)),
            urwid.Text("%s : %s"% (_("Home Directory"), account.homedir)),
            urwid.Text("%s : %s"% (_("Login Shell"), account.shell)),
            widgets.blank,
            urwid.Text(_("Account Maintenance")),
            urwid.GridFlow([btpass], 30, 1, 1, 'left')
            ]
        if widget.label != "root":
            _widgets.append(urwid.GridFlow([btperm], 30, 1, 1, 'left'))
            _widgets.append(urwid.GridFlow([btdel], 30, 1, 1, 'left'))
        
        #self.rightpane = urwid.LineBox(urwid.ListBox(_widgets))
        self.rightpane = urwid.LineBox(
            widgets.ModuleRightPane(parent=self.parent, body=_widgets))
        self.columns.widget_list[-1] = self.rightpane
    
    def do_delete_account(self, widget):
        """ Delete the account after this action has been confirmed """
        try:
            self.usermodel.deleteUser(self.current_user)
            logger.info('Account %s has been deleted'% self.current_user)
        except AssertionError as e:
            dia = dialogs.Error(
                message = _("The following error was encontered while deleting the account") + ":\n\n%s"% e,
                parent = self.parent,
                buttons = [("OK", self.parent.return_to_module)]
                )
            logger.error('Cannot delete account %s'% self.current_user)
            return self.parent.pop_module_dialog(dia, 3)
        dia = dialogs.Info(
            parent = self.parent,
            buttons = [("OK", self.parent.return_to_module)],
            message = _("Account has been deleted and user data has been destroyed.")
            )
        return self.parent.pop_module_dialog(dia, 3)
    
    def confirm_delete_account(self, widget=None):
        """ Ask if you're sure you want to delete this account"""
        dia = dialogs.Question(
            parent = self.parent,
            message = _("Are you sure you want to delete the user account for ") + self.current_user + "?",
            buttons = [("Yes", self.do_delete_account),
                ("No", self.parent.return_to_module_stack)]
            )
        return self.parent.pop_module_dialog(dia, 2)
    
    def prompt_password_change(self, widget=None):
        """ Make a dialog to ask for user's passwords """
        dia = PasswordPrompt(
            parent = self.parent,
            login=self.current_user
        )
        self.parent.pop_module_dialog(dia, 2)
    
    def prompt_user_groups(self, widget=None):
        """ Show a list of groups this user belongs to """
        dia = GroupsPrompt(self, self.usermodel.getUser(self.current_user))
        self.parent.pop_module_dialog(dia, 2)

def __vasm_test__():
    return os.geteuid() == 0

def __run__(parent):
    manager = UserWindow(parent)
    parent.pop_module(manager)
        
