#!/usr/bin/env python

#    This file is part of VASM.
#
#    VASM is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License v3 as published by
#    the Free Software Foundation.
#
#    VASM is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VASM.  If not, see <http://www.gnu.org/licenses/>.

__author__ = "Moises Henriquez"
__author_email__ = "moc.liamg@xnl.E0M"[::-1]

import urwid
import widgets
import os
import sys
if os.getcwd() not in sys.path:
    sys.path.append(os.getcwd())
from tui_utils import _

palette = widgets.palette
ModuleDialog = widgets.TuiModule

class PopupDialog(urwid.Pile):
    def __init__(self, message="", title="MESSAGE", parent=None, buttons = []):
        """ Generic dialog box
        Arguments:
            message = messate to display
            parent = parent widget calling this dialog (not necessaryly the toplevel widget)
            button = list of ('button' callback) tuples
        """
        self.parent = parent
        self.buttons = []
        self._buttons = buttons
        for but, cb in buttons:
            self.buttons.append(
                urwid.AttrWrap(urwid.Button(but, cb), 'button', 'button sel'))
        bar = urwid.GridFlow(self.buttons, 15, 3, 1, 'center')
        
        _list = [
            ('flow', urwid.LineBox(urwid.Text(title, 'right'))),
            ('flow', urwid.Text(message)),
            urwid.SolidFill(),
            ('flow', urwid.Divider('_')),
            ('flow', widgets.blank),
            ('flow', bar)]
        urwid.Pile.__init__(self, _list)
    
    def keypress(self, size, key):
        """ react to 'q' or 'esc' key strokes """
        if key in ('q', 'esc'):
            if len(self._buttons) == 1:
                return self._buttons[0][1]()
            else:
                for bt, cb in self._buttons:
                    if bt in ("Cancel", "cancel", "NO", "no", "No"):
                        return cb()
        else:
            return super(PopupDialog, self).keypress(size, key)
    
class Info(PopupDialog):
    """ preset informational message"""
    def __init__(self, message="", parent=None, buttons=[]):
        PopupDialog.__init__(self, message=message,
                             title="INFORMATION",
                             parent=parent,
                             buttons=buttons)
    
class Error(PopupDialog):
    """ Preset error message dialog """
    def __init__(self, message="", parent=None, buttons=[]):
        PopupDialog.__init__(self, message=message,
                             title="ERROR",
                             parent=parent,
                             buttons = buttons)

class Question(PopupDialog):
    """ Preset question dialog """
    def __init__(self, message="", parent=None, buttons=[]):
        PopupDialog.__init__(self, message=message,
                             title="QUESTION",
                             parent=parent,
                             buttons = buttons)

class Warning(PopupDialog):
    """ Warning dialog """
    def __init__(self, message="", parent=None, buttons=[]):
        PopupDialog.__init__(
            self, message=message,
            title=_("WARNING"),
            parent = parent,
            buttons = buttons
        )

class oldModuleDialog(urwid.Frame):
    """ Module dialog using a frame.
    
    Arguments:
        header: string or urwid widget to be used as a header.
        desc:   List of strings or widgets to be packed below the header line,
            but still into the header section.  This allows scrolling action in
            the body section while keeping the header static.
        body:   list of widets to be packed into the body of the module display.
        buttons:    list of (button, callback) tuples.
        parent:     parent container widget calling this class.
    
    """
    def __init__(self, header="", desc=[], body=None, buttons=[], parent=None):
        self.desc = desc
        self.buttons = buttons
        self.parent = parent
        self.statusbar = urwid.Text("")
        _header = []
        if type(header) == str:
            _header.append(urwid.AttrWrap(urwid.LineBox(urwid.Text(header)), 'header'))
        else:
            _header.append(header)
        
        # pack the description into the header too
        for item in desc:
            if type(item) == str:
                _header.append(urwid.Text(item))
            else:
                _header.append(item)
        self.header = urwid.Pile(_header)
        self._body = []
        if isinstance(body, urwid.Pile):
            pile = body
        else:
            for item in body:
                self._body.append(item)
            _list = [
                urwid.ListBox(self._body),
            ]
            pile = widgets.FrameBody(_list)
        self.footer = self._get_footer()
        urwid.Frame.__init__(
            self,
            body=pile,
            header=self.header,
            footer=self.footer, focus_part='footer')
    
    def pop_dialog(self, dialog, level=1):
        """ Send a dialog to the parent so it can be popped up"""
        return self.parent.pop_module_dialog(dialog, level)

    def pause_ui(self):
        """ Pause execution of the main loop and hand control to the tty """
        ml = self.parent._mainloop
        return ml.screen.stop()

    def resume_ui(self):
        """ Resume execution of the main loop """
        ml = self.parent._mainloop
        return ml.screen.start()
        
    
    
    def _emulate_left_button(self, times=1):
        """ emulate a left arrow hit """
        ml = self.parent._mainloop
        for x in range(0, times):
            ml.process_input(['left'])
            self.keypress((10, 2), 'left')

    def return_to_module(self, caller_widget=None):
        """ Close all dialogs and return to this module's home """
        return self.parent.return_to_module(caller_widget)
    
    def close_module(self, caller_widget=None):
        """ Close this module and return home """
        return self.parent.return_home(caller_widget)

    def release_focus_to_parent(self):
        """ Return focus to the left column"""
        return self._emulate_left_button(2)
    
    def _trigger_cancel_method(self, widget=None):
        for bt, cb in self.buttons:
            if bt in ("Cancel", "Done", "Exit", "Back", "back",
                      "cancel", "done", "exit", "Home", "home"):
                return cb(widget)
        # Look for another way out.
        return self.release_focus_to_parent()
    
    def keypress(self, size, key):
        """ Handle the keys to change focus """
        if key == 'tab':
            if self.focus_part == 'body':   
                self.set_focus('footer')
            else:
                self.set_focus('body')
        elif key in ("esc", "b"):
            return self._trigger_cancel_method()
        
        elif key in ("page up", "page down"):
            # pass it to the body
            if type(self.get_body()) == urwid.Pile:
                return self.get_body().keypress(size, key)
        
        return super(ModuleDialog, self).keypress(size, key)
    

    def set_status_text(self, text):
        """ Set the status text for this module """
        return self.statusbar.set_text(text)
    
    def _get_footer(self):
        """ Return a pile that can be  used for the window's footer"""
        buttons = self._get_action_area()
        statusline = [self.statusbar, buttons]
        return urwid.Columns(statusline)
    
    def _get_action_area(self):
        buttons = []
        for but, cb in self.buttons:
            buttons.append(urwid.LineBox(
                urwid.AttrWrap(
                    urwid.Button(but, cb), 'button', 'button sel'
                ))
            )
        bar = urwid.GridFlow(buttons, 15, 0, 0, 'right')
        return bar
    
    def add_button(self, button, callback):
        """ Add a button to the action area """
        self.buttons.append((button, callback))
        self.footer = self._get_footer()

class VChoiceBox(ModuleDialog):
    def __init__(self, title="", options=[], parent=None, on_select=None):
        self.parent = parent
        self.on_select = on_select
        _grp = []
        _selectors = []
        for item in options:
            rb = urwid.RadioButton(group=_grp, label=item,
                                   on_state_change = self._radio_toggle)
            wrap = urwid.AttrWrap(rb, 'button', 'button sel')
            _selectors.append(wrap)
        _body = _selectors
        ModuleDialog.__init__(self,
                header = urwid.AttrWrap(urwid.Text(title), 'column header'),
            desc = [widgets.blank],
            body = _body,
            buttons = [("OK", self.make_choice),
                           ("Cancel", self.return_to_module)],
        parent = parent)
        self.set_focus('body')
        self.choices = _selectors
        self._selection = None

    def _radio_toggle(self, radio=None, new_state = None, userdata=None):
        if new_state:
            self._selection = radio.get_label()

    def _get_action_area(self):
        buttons = []
        for but, cb in self.buttons:
            buttons.append(
                urwid.AttrWrap(
                    urwid.Button(but, cb), 'button', 'button sel'
                    )
                )

        bar = urwid.GridFlow(buttons, 12, 1, 0, 'right')
        return bar

    def make_choice(self, widget=None):
        if self.on_select:
            self.on_select(self._selection)
        return self.return_to_module()
