#!/usr/bin/env python

#    This file is part of VASM.
#
#    VASM is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License v3 as published by
#    the Free Software Foundation.
#
#    VASM is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VASM.  If not, see <http://www.gnu.org/licenses/>.

__author__ = "Moises Henriquez"
__author_email__ = "moc.liamg@xnl.E0M"[::-1]

import urwid
import calendar
import datetime
palette = [
        ('body','white','black', 'standout'),
        ('dialog body', 'white', 'dark gray', 'standout'),
        ('border','black','dark blue'),
        ('selectable','black', 'dark cyan'),
        ('focus','white','dark gray','bold'),
        ('focustext','light gray','dark blue'),
        ('header', 'white', 'light blue', 'bold'),
        ('reveal focus', 'black', 'dark cyan', 'standout'),
        ('shadow',  'dark gray',  'dark gray'),
        ('shadow text', 'white', 'dark gray', 'standout'),
        ('button sel', 'black', 'dark cyan'),
        ('button', 'dark cyan', 'black'),
        ('entry done', 'black', 'light gray'),
        ('entry', 'light gray', 'black', 'standout'),
        ('warning', 'white', 'dark red', ('bold', 'standout')),
        ('column header', 'black','light gray'),
        ]

tui_palette = [
        ('body', 'white', 'black'),
        ('dialog body', 'black', 'light gray', 'standout'),
        ('border', 'black', 'dark blue'),
        ('selectable', 'black', 'dark cyan'),
        ('focus', 'black', 'light gray', ('bold', 'standout')),
        ('focustext', 'light gray', 'dark blue'),
        ('header', 'black', 'light gray', ('bold', 'standout')),
        ('reveal focus', 'dark blue', 'dark green', 'standout'),
        ('shadow', 'light gray', 'light gray'),
        ('shadow text', 'dark blue', 'light gray', 'standout'),
        ('button', 'dark cyan', 'black', ('bold', 'standout')),
        ('button sel', 'white', 'dark blue'),
        ('entry done', 'black', 'light gray'),
        ('entry', 'light gray', 'black', 'standout'),
        ('warning', 'white', 'dark red', ('bold', 'standout')),
        ('column header', 'black','light gray'),
        ]

blank = urwid.Divider()

class ModuleRightPane(urwid.ListBox):
    """ Listbox to be used in a module as the right pane.  Based on ListBox
    only has the esc key mapped to exit to the left pane."""
    
    def __init__(self, body, parent=None):
        self.body = body
        self.parent = parent
        urwid.ListBox.__init__(self, self.body)

    def _emulate_left_button(self, times=1):
        if not self.parent:
            return
        ml = self.parent._mainloop
        for x in range(0, times):
           ml.process_input(['left'])
           self.keypress((10, 2), 'left')

class CategoryDisplay(urwid.SimpleListWalker):
    """ Custom list walker used to display settings categories.
    
    Modifications make it indicate which child widget was selected even
    after the parent widget itself has lost focus
    
    * Holds many CategoryLabel objects
    """
    def __init__(self, widget_list):
        self._widgets = widget_list
        urwid.SimpleListWalker.__init__(self, widget_list)
    
    def get_focus(self):
        curr, pos = super(CategoryDisplay, self).get_focus()
        cw = urwid.AttrWrap(curr, "focus")
        return cw, pos

    def keypress(self, size, key):
        if key in ('Q', 'q'):
            raise urwid.ExitMainLoop
        
        for child in self.widgets:
            return child.keypress(size, key)
        return super(CategoryDisplay, self).keypress(self, key)

class SensibleCheckBox(urwid.CheckBox):
    """ Checkbox that can trigger a method when it receives focus """
    def __init__(self, label, state=False,
                 has_mixed=False, on_state_change=None,user_data=None,
                 on_focus=None):
        self.user_data = user_data
        self.on_focus = on_focus
        urwid.CheckBox.__init__(self,
                                label = label,
                                state = state,
                                has_mixed = has_mixed,
                                on_state_change = on_state_change,
                                user_data = user_data)
        
    def render(self, size, focus=False):
        """ Use this method to trigger the focus change event """
        if focus:
            if self.on_focus:
                self.on_focus(self)
        return super(SensibleCheckBox, self).render(size, focus)

class SensibleRadioButton(urwid.RadioButton):
    """ Radio button widget that can trigger a method when it receives focus """
    def __init__(self, label="", group=None, state=False, on_state_change=None,
        user_data=None, on_focus=None):
        self.on_focus=on_focus
        self.user_data = user_data
        urwid.RadioButton.__init__(
            self,
            group,
            label,
            state,
            on_state_change,
            user_data
            )
    def selectable(self):
        return True

    def render(self, size, focus=False):
        if focus:
            if self.on_focus:
                self.on_focus(self)
        return super(SensibleRadioButton, self).render(size, focus)

class CategoryLabel(urwid.Text):
    """ Selectable label used to represent settings categories
    Arguments:
        markup: Text to be used as the label's markup
        focus_method: Method to be executed when the object receives focus
                        Hint: Use this to refresh this category's child widgets
    * Is child to CategoryDisplay object
    """
    def __init__(self, markup, focus_method, enter_sensitive=False, parent=None):
        self.focusmethod = focus_method
        self._select_method = None
        self.parent = parent
        self.enter_sensitive = enter_sensitive
        self.label = markup
        urwid.Text.__init__(self, markup)
    
    def selectable(self):
        """ Override the selectable property to make sure we maintain some
        type of indicator even after the parent has lost focus """
        return True

    def render(self, size, focus=False):
        """ Indicate the status of this category even after the parent has lost focus """
        if focus:
            if self.focusmethod:
                self.focusmethod(self)
        
        return super(CategoryLabel, self).render(size, focus)

    def keypress(self, size, key):
        """ Return the key that was pressed over to the parent"""
        if key in ("enter", ' '):
            if self.enter_sensitive:
                if self.parent:
                    ml = self.parent._mainloop
                    ml.process_input(['right'])
                    self.keypress((10,2), 'right')
                elif self._select_method:
                    return self._select_method(self)
            else:
                if self.parent:
                    ml = self.parent._mainloop
                    ml.process_input(['right'])
                    self.keypress((10,2), 'right')
        return key

class Launcher(urwid.Text):
    """ Selectable text label widget used to represent a launcher.  Each launcher
    represents a settings option in vasm.
    Arguments:
        markup: string to be used as the label's markup text
        category:   String used to indicate which category this label belongs to
        actuator:   method to be executed when this launcher is selected by pressing
                    the 'enter' key
    """
    def __init__(self, markup="", category="", actuator=None, parent=None):
        self.parent = parent
        urwid.Text.__init__(self, markup)
        self.actuator = actuator
        self.category = category
        
    def selectable(self):
        return True

    def keypress(self, size, key):
        """ Process key presses to trigger launcher """
        if key in ('enter', ' '):
            return self.actuator(self.parent)
        elif key == "esc":
            ml = self.parent._mainloop
            ml.process_input(['left'])
            self.keypress((10, 2), 'left')
        return key

class SensibleLabel(urwid.Text):
    """ Selectable label widget that can trigger a method when it receives focus"""
    def __init__(self, markup="", actuator=None, user_data=None, on_focus=None):
        urwid.Text.__init__(self, markup)
        self.on_focus = on_focus
        self.actuator = actuator
        self.user_data = user_data

    def selectable(self):
        return True

    def render(self, size, focus=False):
        if focus:
            if self.on_focus:
                self.on_focus(self)
        return super(SensibleLabel, self).render(size, focus)

    def keypress(self, size, key):
        if key in ('enter', ' '):
            if self.actuator:
                return self.actuator(self)
        return key

    def set_value(self, value):
        return self.set_text(value)

    def get_value(self):
        return self.get_text()[0]

class FrameBody(urwid.Pile):
    """ Widget to be used as a frame's body widget with special key bindings
    for the tab key """
    def __init__(self, widget_list, focus_item=None):
        urwid.Pile.__init__(self, widget_list, focus_item)

class DayLb(urwid.Text):
    """ Selectable text label used to represent a day in the calendar widget """
    def __init__(self, markup="", on_select=None, user_data = None):
        self.on_select = on_select
        self.user_data = user_data
        urwid.Text.__init__(self, markup, align='left')
    
    def selectable(self):
        return True

    def keypress(self, size, key):
        if self.get_text == " ":
            return
        return key
    
    def render(self, size, focus=False):
        """ Trigger the selected event """
        if focus:
            if self.on_select:
                self.on_select(self, self.user_data)
        return super(DayLb, self).render(size, focus)

class Calendar(urwid.Pile):
    """ Calendar widget """
    def __init__(self, year=2011, month=3):
        self._header = urwid.Columns(
            [urwid.AttrWrap(urwid.Text(t), 'column header')
            for t in calendar.weekheader(3).split()], 1, 1)
        self.monthline = urwid.Text("", align='right')
        self._footer = urwid.Text("", align='right')
        self.selected_date = None
        self._widgets = []
        now = datetime.datetime.now()
        self._footer.set_text("No date is selected")
        urwid.Pile.__init__(self, self._widgets)
        # set the view to the current month
        self.set_month(now.year, now.month)
        self._highlight_day(now.year, now.month, now.day)
    
    def _highlight_day(self, year, month, day):
        rows = [row for row in self._widgets if type(row) is urwid.Columns]
        irow = 0
        for row in rows:
            icol = 0
            for col in row.widget_list:
                if col.text == str(day):
                    row.set_focus_column(icol)
                    data = col.user_data
                    self.set_date((data[0], data[1], col.text))
                    self.selected_date = ((year, month, day))
                    return
                icol = icol + 1
            irow = irow + 1
    
    def set_month(self, year, month):
        self.cal = calendar.Calendar()
        self.currmonth = self.cal.monthdatescalendar(year, month)
        self._year = year
        self._month = month
        # set the month name
        self.monthline.set_text(calendar.month_name[month] + ", " + str(year))
        self._widgets = [
            self.monthline,
            self._header,
        ]
        # create the week columns.
        for week in self.currmonth:
            _week_widgets = []
            for date in week:
                w = DayLb(str(date.day), on_select = self.update_selection,
                          user_data = (date.year, date.month))
                if date.month != month:
                    # make an old widget.
                    wrap = urwid.AttrWrap(w, None, 'shadow text')
                else:
                    # make current month's widget
                    wrap = urwid.AttrWrap(w, None, 'reveal focus')
                _week_widgets.append(wrap)
            row = urwid.Columns(_week_widgets, 1, 1)
            self._widgets.append(row)
        
        # append the footer
        self._widgets.append(blank)
        self._widgets.append(self._footer)
        self.widget_list[:] = self._widgets
        self.set_focus(3)
    
    def update_selection(self, widget, user_data):
        dyr, dmo = user_data
        self._selected_date = ((dyr, dmo, int(widget.text)))
    
    def set_date(self, date_tuple):
        self.selected_date = date_tuple
        self._footer.set_text("Selected Date: %s-%s-%s "% date_tuple)
    
    def _get_prev_month(self, curryr, currmo):
        """ Get the previous month """
        if currmo == 1:
            return (curryr - 1, 11)
        else:
            return (curryr, (currmo - 1))
    
    def _get_next_month(self, curryr, currmo):
        if currmo == 11:
            return ((curryr + 1), 1)
        else:
            return (curryr, (currmo + 1))
    
    def get_date(self):
        """ Returns the selected date in a tupe (year, month, day) """
        return self.selected_date
    
    def keypress(self, size, key):
        if key in ('page up', '-'):
            nyr, nmo = self._get_prev_month(self._year, self._month)
            self.selected_date = None
            self._footer.set_text("No date selected ")
            return self.set_month(nyr, nmo)
        elif key in ('page down', '+'):
            nyr, nmo = self._get_next_month(self._year, self._month)
            self.selected_date = None
            self._footer.set_text("No date selected ")
            return self.set_month(nyr, nmo)
        
        elif key in (' ', 'enter'):
            return self.set_date(self._selected_date)

        return super(Calendar, self).keypress(size, key)

class TimeSpinner(urwid.Text):
    """ Selectable time spinner"""
    def __init__(self, markup="0", rangemax=24, on_focus=None):
        """ change rangemax to 59 for a minutes/seconds picker """
        self.on_focus = on_focus
        urwid.Text.__init__(self, markup)
        self.range = range(0, rangemax)
    
    def selectable(self):
        return True

    def _go_next(self):
        currval = int(self.text)
        if currval == self.range[-1]:
            newval = 0
        else:
            newval = self.range.index(int(self.text) + 1)
        self.set_text(str(newval))
    
    def _go_back(self):
        currval = int(self.text)
        if currval == self.range[0]:
            newval = self.range[-1]
        else:
            newval = self.range.index(int(self.text) - 1)
        
        self.set_text(str(newval))
    
    def keypress(self, size, key):
        if key in ("-", 'page down'):
            return self._go_back()
        elif key in ("+", 'page up'):
            return self._go_next()
        return key

    def get_value(self):
        return int(self.text)

    def render(self, size, focus=False):
        if self.on_focus:
            self.on_focus(self)
        return super(TimeSpinner, self).render(size, focus)

class LabeledSpinner(urwid.Columns):
    def __init__(self, label="", default_value=0, range_max=60, on_focus=None):
        self.label = urwid.Text(label)
        self.spinner = TimeSpinner(markup=str(default_value),rangemax = range_max,
                       on_focus=on_focus)
        urwid.Columns.__init__(
            self,[
            ('fixed', len(label) + 1, self.label),
            ('fixed', 4,
             urwid.AttrWrap(self.spinner, 'button', 'button sel'))])

    def get_value(self):
        return self.spinner.get_value()

    def set_value(self, value):
        return self.spinner.set_value(float(value))

    def set_label(self, markup):
        return self.label.set_text(markup)

class TimePicker(urwid.Pile):
    def __init__(self):
        now = datetime.datetime.now()
        self.hrpicker = TimeSpinner(str(now.hour))
        self.mnpicker = TimeSpinner(str(now.minute),rangemax=60)
        self.scpicker = TimeSpinner(str(now.second),rangemax=60)
        
        hdr = urwid.Text(
            "Use +/- to increase / decrease the value of the selected spinner."
        )
        hrt = urwid.Text("Hours: ")
        mnt = urwid.Text("Minutes: ")
        sct = urwid.Text("Seconds: ")
        whr = urwid.AttrWrap(self.hrpicker, 'shadow text', 'reveal focus')
        wmn = urwid.AttrWrap(self.mnpicker, 'shadow text', 'reveal focus')
        wsc = urwid.AttrWrap(self.scpicker, 'shadow text', 'reveal focus')
        row = urwid.Columns([
            ('fixed', 6, hrt),
            ('fixed', 3, whr),
            ('fixed', 8, mnt),
            ('fixed', 3, wmn),
            ('fixed', 8, sct),
            ('fixed', 3, wsc)],
             1, 1)
        
        _widgets = [
            ('flow', blank),
            ('flow', hdr),
            ('flow', blank),
            ('flow', row)
        ]
        urwid.Pile.__init__(self, _widgets)
    
    def set_time(self, time_tuple):
        """ Set the time on the spinners to the specified values in the tuple """
        hour, minute, second = time_tuple
        self.hrpicker.set_text(str(hour))
        self.mnpicker.set_text(str(minute))
        self.scpicker.set_text(str(second))
    
    def get_time(self):
        """ Returns the set time in a tuple (hours, minuts, seconds) """
        return (int(self.hrpicker.text),
                int(self.mnpicker.text),
                int(self.scpicker.text))


class HChoiceBox(urwid.Columns):
    def __init__(self, label="", choices = [], on_state_change=None,
            on_focus=None):
        self.on_focus=on_focus
        self.label = urwid.Text(label)
        self.on_state_change = on_state_change
        rbgroup = []
        _widgets = [('fixed', len(label) + 1, self.label)]
        rbw = 0
        self._radios = []
        for item, data in choices:
            rbw = max(rbw, len(item) + 6, 10)
            rb = SensibleRadioButton(group=rbgroup, label=item,
                     on_state_change=self.radio_toggle,
                     user_data = data,
                     on_focus=on_focus)
                #rb = urwid.RadioButton(group=rbgroup, label=item, on_state_change=self.radio_toggle)
            self._radios.append(rb)
            wrap = urwid.AttrWrap(rb, 'button','button sel')
            _widgets.append(('fixed', rbw, wrap))
        urwid.Columns.__init__(self, _widgets)

    def radio_toggle(self, radio=None, new_value=None, user_data=None):
        if self.on_state_change:
            self.on_state_change(radio, new_value, user_data)
        if new_value:
            self._selection = radio.get_label()
        return

    def get_value(self):
        return self._selection

    def get_value_data(self):
        for rb in self._radios:
            if rb.get_label() == self._selection:
                return rb.user_data

    def set_value(self, value):
        for item in self._radios:
            if item.get_label().upper() == value.upper():
                return item.toggle_state()

class TuiModule(urwid.Frame):
    """ Module dialog using a frame.
    
    Arguments:
        header: string or urwid widget to be used as a header.
        desc:   List of strings or widgets to be packed below the header line,
            but still into the header section.  This allows scrolling action in
            the body section while keeping the header static.
        body:   list of widets to be packed into the body of the module display.
        buttons:    list of (button, callback) tuples.
        parent:     parent container widget calling this class.
    
    """
    def __init__(self, header="", desc=[], body=None,
            buttons=[], parent=None):
        self.desc = desc
        self.buttons = buttons
        self.parent = parent
        self.statusbar = urwid.Text("")
        _header = []
        if type(header) == str:
            _header.append(urwid.AttrWrap(
                urwid.LineBox(urwid.Text(header)), 'header'))
        else:
            _header.append(header)
        
        # pack the description into the header too
        for item in desc:
            if type(item) == str:
                _header.append(urwid.Text(item))
            else:
                _header.append(item)
        self.header = urwid.Pile(_header)
        self._body = []
        if isinstance(body, urwid.Widget):#urwid.Pile):
            pile = body
        else:
                for item in body:
                    self._body.append(item)
                _list = [
                    urwid.ListBox(self._body),
                ]
                pile = FrameBody(_list)
        self.footer = self._get_footer()
        urwid.Frame.__init__(
                self,
                body=pile,
                header=self.header,
                footer=self.footer, focus_part='footer')
    
    def pop_dialog(self, dialog, level=1):
        """ Send a dialog to the parent so it can be popped up"""
        return self.parent.pop_module_dialog(dialog, level)

    def pause_ui(self):
        """ Pause execution of the main loop and hand control to the tty """
        ml = self.parent._mainloop
        return ml.screen.stop()

    def resume_ui(self):
        """ Resume execution of the main loop """
        ml = self.parent._mainloop
        return ml.screen.start()
    
    def _emulate_left_button(self, times=1):
        """ emulate a left arrow hit """
        ml = self.parent._mainloop
        for x in range(0, times):
            ml.process_input(['left'])
            self.keypress((10, 2), 'left')

    def return_to_module(self, caller_widget=None, data=None):
        """ Close all dialogs and return to this module's home """
        return self.parent.return_to_module(caller_widget)
    
    def close_module(self, caller_widget=None, data=None):
        """ Close this module and return home """
        return self.parent.return_home(caller_widget, data)

    def release_focus_to_parent(self):
        """ Return focus to the left column"""
        return self._emulate_left_button(2)
    
    def _trigger_cancel_method(self, widget=None):
        for entry in self.buttons:
            if len(entry) == 2:
                bt = entry[0]
                cb = entry[1]
                data = None
            elif len(entry) == 3:
                bt =entry[0]
                cb = entry[1]
                data = entry[2]
        if bt in ("Cancel", "Done", "Exit", "Back", "back","cancel",
            "home", "done", "Home"):
            return cb(bt, data)
        # Look for another way out.
        return self.release_focus_to_parent()
    
    def keypress(self, size, key):
        """ Handle the keys to change focus """
        if key == 'tab':
            if self.focus_part == 'body':   
                self.set_focus('footer')
            else:
                self.set_focus('body')
        elif key in ("esc",):
            return self._trigger_cancel_method()
        
        elif key in ("page up", "page down"):
            # pass it to the body
            if type(self.get_body()) == urwid.Pile:
                return self.get_body().keypress(size, key)
        return super(TuiModule, self).keypress(size, key)
    

    def set_status_text(self, text):
        """ Set the status text for this module """
        return self.statusbar.set_text(text)
    
    def _get_footer(self):
        """ Return a pile that can be  used for the window's footer"""
        buttons = self._get_action_area()
        statusline = [self.statusbar, buttons]
        return urwid.Columns(statusline)
    
    def _get_action_area(self):
        buttons = []
        for setup in self.buttons:
            if len(setup) == 2:
                lb, cb = setup
                buttons.append(urwid.LineBox(
                    urwid.AttrWrap(
                        urwid.Button(lb, cb, None),'button', 'button sel')))
            else:
                lb, cb, data = setup
                buttons.append(urwid.LineBox(
                    urwid.AttrWrap(
                        urwid.Button(lb, cb, data), 'button', 'button sel')))

        bar = urwid.GridFlow(buttons, 15, 0, 0, 'right')
        if buttons:
            bar.set_focus(buttons[0])
        return bar
    
    def add_button(self, button, callback, data=None):
        """ Add a button to the action area """
        self.buttons.append((button, callback, data))
        self.footer = self._get_footer()

class VChoicePopUp(TuiModule):
    def __init__(self, title="", options=[], parent=None, on_select=None):
        self.parent = parent
        self.on_select = on_select

        TuiModule.__init__(self,
            header = urwid.AttrWrap(urwid.Text(title), 'column header'),
            desc = [blank],
            body = [blank],
            buttons = [("OK", self.make_choice),
                        ("Cancel", self.return_to_module)],
            parent = parent)
        self.set_options(options)
        self.set_focus('body')
        if self.choices:
            self._selection = self.choices[0].original_widget.get_label()

    def set_options(self, options):
        _grp = []
        _selectors = []
        for item in options:
            rb = urwid.RadioButton(group = _grp,
                           label = item,
                on_state_change = self._radio_toggle)
            wrap = urwid.AttrWrap(rb, 'button', 'button sel')
            _selectors.append(wrap)
        self.choices = _selectors
        self.set_body(urwid.ListBox(_selectors))

    def _radio_toggle(self, radio=None, new_state = None, userdata=None):
        if new_state:
            self._selection = radio.get_label()

    def _get_action_area(self):
        buttons = []
        for but, cb in self.buttons:
            buttons.append(
                urwid.AttrWrap(
                    urwid.Button(but, cb), 'button', 'button sel'
                    )
                )

        bar = urwid.GridFlow(buttons, 12, 1, 0, 'right')
        return bar

    def make_choice(self, widget=None):
        if self.on_select:
            self.on_select(self._selection)
        return self.return_to_module(None, None)


class LabeledPicker(urwid.Columns):
    def __init__(self, label="", picker_title = "", options = [],
            parent = None, on_focus=None):
        self._default_value = "[ Select ]"
        self.on_focus = on_focus
        self._options = options
        self.parent = parent
        self.title = picker_title
        self._label = label
        self.label = urwid.Text(label)
        self.selector = SensibleLabel(
            markup = self._default_value,
            actuator = self._launch_picker,
            on_focus = self.on_focus,
            )
        selwrap = urwid.AttrWrap(self.selector, 'button', 'button sel')
        # the picker
        self.picker = VChoicePopUp(
            parent = self.parent,
            title = self.title,
            options = self._options,
            on_select = self.set_value
            )
        # The width of the first column should be == len(self._label)
        urwid.Columns.__init__(self,
                       [('fixed', len(self._label) + 1, self.label),
                       selwrap]
                       )

    def reset_value(self):
        """ Reset the value for this picker"""
        return self.selector.set_value(self._default_value)

    def set_options(self, options):
        """ Modify the provided options """
        return self.picker.set_options(options)

    def _launch_picker(self, caller_widget=None):
        """ Launch the picker widget to let the user select."""
        return self.parent.pop_dialog(self.picker)

    def get_value(self):
        """ Read the current value of the picker """
        return self.selector.get_text()[0]

    def set_value(self, value):
        """ Set the value to the picker """
        return self.selector.set_text(value)

class GridRow(urwid.Columns):
    def __init__(self, columns=[], tag="", parent=None):
        assert isinstance(parent, GridView), 'Parent must be GridView object.'
        assert len(parent._columns) == len(columns), \
            'Different number of columns between parent and child'
        self.tag = self.id = tag
        self._columns = columns
        self.length = len(self._columns)
        _widgets = []
        _widths = parent._column_widths
        i = 0
        for item in columns:
            _width = _widths[i]
            if isinstance(item, urwid.Widget):
                w = item
            elif type(item) is str:
                w = urwid.Text(item)
            if _width > 0:
                _widgets.append(('fixed', _width, w))
            else:
                _widgets.append(w)
            i += 1
        urwid.Columns.__init__(self, _widgets, dividechars=parent._divider_space)

class GridView(urwid.Frame):
    def __init__(self, columns=[], divspace=0):
        self._columns = columns
        self._divider_space = divspace
        self._colwidgets = []
        self._rows = []
        self._column_widths = []
        for col, width in columns:
            wrap = urwid.AttrWrap(urwid.Text(col), 'column header')
            if width == 0: # Automatic
                self._colwidgets.append(wrap)
                self._column_widths.append(0)
            elif width == -1: # width = size of text
                self._colwidgets.append(('fixed', len(col) + 1, wrap))
                self._column_widths.append(len(col) + 1)
            else: # custom width - as specified 
                self._colwidgets.append(('fixed', width, wrap))
                self._column_widths.append(width)
        self.headers = urwid.Columns(self._colwidgets, dividechars=self._divider_space)
        header = urwid.Pile([('flow', self.headers)])
        self._widgets = urwid.SimpleListWalker([])
        self._body = urwid.ListBox(self._widgets)
        urwid.Frame.__init__(self,
                    header = header,
                    body = self._body)

    def append_row(self, row):
        """ Append a row to the gridview """
        assert isinstance(row, GridRow), 'row argument must be GridRow instance'
        self._rows.append(row)
        return self._widgets.append(row)

    def clear(self):
        self._body.body[:] = urwid.SimpleListWalker([])
        return
